@@ -34,6 +34,12 @@
 #include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/uio.h>
+#if defined(__sun)
+#include <sys/filio.h>
+#if !defined(MAX)
+#define MAX(a, b) (((a) > (b)) ? (a) : (b))
+#endif
+#endif
 #include <errno.h>
 #include <arpa/inet.h>
 #include <net/if.h>
@@ -176,10 +182,6 @@ SockadrToNetadr(struct sockaddr_storage *s, netadr_t *a)
 			a->scope_id = s6->sin6_scope_id;
 		}
 	}
-	else
-	{
-		s = NULL;
-	}
 }
 
 void
@@ -273,7 +275,7 @@ NET_CompareBaseAdr(netadr_t a, netadr_t b)
 char *
 NET_BaseAdrToString(netadr_t a)
 {
-	static char s[64], tmp[64];
+	static char s[64];
 	struct sockaddr_storage ss;
 	struct sockaddr_in6 *s6;
 
@@ -332,6 +334,8 @@ NET_BaseAdrToString(netadr_t a)
 					IN6_IS_ADDR_LINKLOCAL(&((struct sockaddr_in6 *)&ss)->
 								sin6_addr))
 				{
+					char tmp[64];
+
 					/* If the address is multicast (link) or a
 					   link-local, need to carry the scope. The string
 					   format of the IPv6 address is used by the
@@ -366,7 +370,7 @@ NET_AdrToString(netadr_t a)
 }
 
 qboolean
-NET_StringToSockaddr(char *s, struct sockaddr_storage *sadr)
+NET_StringToSockaddr(const char *s, struct sockaddr_storage *sadr)
 {
 	char copy[128];
 	char *addrs, *space;
@@ -419,13 +423,8 @@ NET_StringToSockaddr(char *s, struct sockaddr_storage *sadr)
 	switch (resultp->ai_family)
 	{
 		case AF_INET:
-			/* convert to ipv4 addr */
-			memset(sadr, 0, sizeof(struct sockaddr_storage));
-			memcpy(sadr, resultp->ai_addr, resultp->ai_addrlen);
-			break;
-
 		case AF_INET6:
-			/* convert to ipv6 addr */
+			/* convert to ipv4/ipv6 addr */
 			memset(sadr, 0, sizeof(struct sockaddr_storage));
 			memcpy(sadr, resultp->ai_addr, resultp->ai_addrlen);
 			break;
@@ -436,11 +435,13 @@ NET_StringToSockaddr(char *s, struct sockaddr_storage *sadr)
 			return 0;
 	}
 
+	freeaddrinfo(resultp);
+
 	return true;
 }
 
 qboolean
-NET_StringToAdr(char *s, netadr_t *a)
+NET_StringToAdr(const char *s, netadr_t *a)
 {
 	struct sockaddr_storage sadr;
 
@@ -660,11 +661,13 @@ NET_SendPacket(netsrc_t sock, int length, void *data, netadr_t to)
 		{
 			struct addrinfo hints;
 			struct addrinfo *res;
-			char tmp[128], mcast_addr[128], mcast_port[10];
-			int error;
+			char tmp[128];
 
 			if (multicast_interface != NULL)
 			{
+				int error;
+				char mcast_addr[128], mcast_port[10];
+
 				/* Do a getnameinfo/getaddrinfo cycle
 				   to calculate the scope_id of the
 				   multicast address. getaddrinfo is
@@ -706,6 +709,7 @@ NET_SendPacket(netsrc_t sock, int length, void *data, netadr_t to)
 
 				/* sockaddr_in6 should now have a valid scope_id. */
 				memcpy(s6, res->ai_addr, res->ai_addrlen);
+				freeaddrinfo(res);
 			}
 			else
 			{
@@ -769,10 +773,10 @@ NET_OpenIP(void)
 void
 NET_Config(qboolean multiplayer)
 {
-	int i;
-
 	if (!multiplayer)
 	{
+		int i;
+
 		/* shut down any existing sockets */
 		for (i = 0; i < 2; i++)
 		{
@@ -808,7 +812,8 @@ int
 NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 {
 	char Buf[BUFSIZ], *Host, *Service;
-	int newsocket, Error;
+	int newsocket = 0;
+	int Error = 0;
 	struct sockaddr_storage ss;
 	struct addrinfo hints, *res, *ai;
 	qboolean _true = true;
@@ -845,13 +850,14 @@ NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 
 	if ((Error = getaddrinfo(Host, Service, &hints, &res)))
 	{
+		Com_Printf("ERROR: NET_Socket: getaddrinfo:%s\n",
+				gai_strerror(Error));
 		return 0;
 	}
 
 	for (ai = res; ai != NULL; ai = ai->ai_next)
 	{
-		if ((newsocket =
-				 socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) == -1)
+		if ((newsocket = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) == -1)
 		{
 			Com_Printf("NET_Socket: socket: %s\n", strerror(errno));
 			continue;
@@ -861,6 +867,7 @@ NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 		if (ioctl(newsocket, FIONBIO, (char *)&_true) == -1)
 		{
 			Com_Printf("NET_Socket: ioctl FIONBIO: %s\n", strerror(errno));
+			close(newsocket);
 			continue;
 		}
 
@@ -872,6 +879,8 @@ NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 			{
 				Com_Printf("ERROR: NET_Socket: setsockopt SO_BROADCAST:%s\n",
 						NET_ErrorString());
+				freeaddrinfo(res);
+				close(newsocket);
 				return 0;
 			}
 		}
@@ -882,12 +891,15 @@ NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 		{
 			Com_Printf("ERROR: NET_Socket: setsockopt SO_REUSEADDR:%s\n",
 					NET_ErrorString());
+			freeaddrinfo(res);
+			close(newsocket);
 			return 0;
 		}
 
 		if (bind(newsocket, ai->ai_addr, ai->ai_addrlen) < 0)
 		{
 			Com_Printf("NET_Socket: bind: %s\n", strerror(errno));
+			close(newsocket);
 		}
 		else
 		{
@@ -903,6 +915,7 @@ NET_Socket(char *net_interface, int port, netsrc_t type, int family)
 
 	if (ai == NULL)
 	{
+		close(newsocket);
 		return 0;
 	}
 
