@@ -19,56 +19,42 @@
  *
  * =======================================================================
  *
- * This file is the starting point of the program and implements
- * several support functions and the main loop.
+ * This file implements all system dependent generic functions.
  *
  * =======================================================================
  */
 
+#include <conio.h>
+#include <direct.h>
 #include <errno.h>
 #include <float.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <direct.h>
 #include <io.h>
-#include <conio.h>
 #include <shlobj.h>
+#include <stdio.h>
+#include <wchar.h>
+#include <windows.h>
 
 #include "../../common/header/common.h"
-#include "../generic/header/input.h"
 #include "header/resource.h"
-#include "header/winquake.h"
-
-#define MAX_NUM_ARGVS 128
-
-int curtime;
-int starttime;
-qboolean ActiveApp;
-qboolean Minimized;
 
+// stdin and stdout handles
 static HANDLE hinput, houtput;
-static HANDLE qwclsemaphore;
 
-HINSTANCE global_hInstance;
+// Game library handle
 static HINSTANCE game_library;
 
-unsigned int sys_msg_time;
-unsigned int sys_frame_time;
+// Config dir
+char cfgdir[MAX_OSPATH] = CFGDIR;
 
+// Buffer for the dedicated server console
 static char console_text[256];
-static int console_textlen;
-
-char findbase[MAX_OSPATH];
-char findpath[MAX_OSPATH];
-int findhandle;
-
-int argc;
-char *argv[MAX_NUM_ARGVS];
+static size_t console_textlen;
 
 /* ================================================================ */
 
 void
-Sys_Error(char *error, ...)
+Sys_Error(const char *error, ...)
 {
 	va_list argptr;
 	char text[1024];
@@ -80,20 +66,13 @@ Sys_Error(char *error, ...)
 	Qcommon_Shutdown();
 
 	va_start(argptr, error);
-	vsprintf(text, error, argptr);
+	vsnprintf(text, sizeof(text), error, argptr);
 	va_end(argptr);
-
-#ifndef DEDICATED_ONLY
 	fprintf(stderr, "Error: %s\n", text);
-#endif
+	fprintf(stdout, "Error: %s\n", text);
 
 	MessageBox(NULL, text, "Error", 0 /* MB_OK */);
 
-	if (qwclsemaphore)
-	{
-		CloseHandle(qwclsemaphore);
-	}
-
 	/* Close stdout and stderr */
 #ifndef DEDICATED_ONLY
 	fclose(stdout);
@@ -113,13 +92,14 @@ Sys_Quit(void)
 #endif
 
 	Qcommon_Shutdown();
-	CloseHandle(qwclsemaphore);
 
 	if (dedicated && dedicated->value)
 	{
 		FreeConsole();
 	}
 
+	printf( "------------------------------------\n" );
+
 	/* Close stdout and stderr */
 #ifndef DEDICATED_ONLY
 	fclose(stdout);
@@ -129,24 +109,6 @@ Sys_Quit(void)
 	exit(0);
 }
 
-void
-WinError(void)
-{
-	LPVOID lpMsgBuf;
-
-	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
-			NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-			(LPTSTR)&lpMsgBuf, 0, NULL);
-
-	/* Display the string. */
-	MessageBox(NULL, lpMsgBuf, "GetLastError", MB_OK | MB_ICONINFORMATION);
-
-	/* Free the buffer. */
-	LocalFree(lpMsgBuf);
-}
-
-/* ================================================================ */
-
 void
 Sys_Init(void)
 {
@@ -165,8 +127,8 @@ Sys_Init(void)
 	   above. Testing older version would be a
 	   PITA. */
 	if (!((vinfo.dwMajorVersion > 5) ||
-		  ((vinfo.dwMajorVersion == 5) &&
-		   (vinfo.dwMinorVersion >= 1))))
+		((vinfo.dwMajorVersion == 5) &&
+			(vinfo.dwMinorVersion >= 1))))
 	{
 		Sys_Error("Yamagi Quake II needs Windows XP or higher!\n");
 	}
@@ -181,6 +143,8 @@ Sys_Init(void)
 	}
 }
 
+/* ================================================================ */
+
 char *
 Sys_ConsoleInput(void)
 {
@@ -272,6 +236,12 @@ Sys_ConsoleOutput(char *string)
 	char text[256];
 	DWORD dummy;
 
+	if ((string[0] == 0x01) || (string[0] == 0x02))
+	{
+		// remove color marker
+		string[0] = ' ';
+	}
+
 	if (!dedicated || !dedicated->value)
 	{
 		fputs(string, stdout);
@@ -296,15 +266,124 @@ Sys_ConsoleOutput(char *string)
 	}
 }
 
-void
-Sys_SendKeyEvents(void)
-{
-#ifndef DEDICATED_ONLY
-	IN_Update();
-#endif
+/* ================================================================ */
 
-	/* grab frame time */
-	sys_frame_time = timeGetTime();
+long long
+Sys_Microseconds(void)
+{
+	static LARGE_INTEGER freq = { 0 };
+	static LARGE_INTEGER base = { 0 };
+
+    if (!freq.QuadPart)
+	{
+		QueryPerformanceFrequency(&freq);
+	}
+
+	if (!base.QuadPart)
+	{
+		QueryPerformanceCounter(&base);
+		base.QuadPart -= 1001;
+	}
+
+	LARGE_INTEGER cur;
+	QueryPerformanceCounter(&cur);
+
+	return (cur.QuadPart - base.QuadPart) * 1000000 / freq.QuadPart;
+}
+
+int
+Sys_Milliseconds(void)
+{
+	return (int)(Sys_Microseconds()/1000ll);
+}
+
+void
+Sys_Nanosleep(int nanosec)
+{
+	HANDLE timer;
+	LARGE_INTEGER li;
+
+	timer = CreateWaitableTimer(NULL, TRUE, NULL);
+
+	// Windows has a max. resolution of 100ns.
+	li.QuadPart = -nanosec / 100;
+
+	SetWaitableTimer(timer, &li, 0, NULL, NULL, FALSE);
+	WaitForSingleObject(timer, INFINITE);
+
+	CloseHandle(timer);
+}
+
+/* ================================================================ */
+
+/* The musthave and canhave arguments are unused in YQ2. We
+   can't remove them since Sys_FindFirst() and Sys_FindNext()
+   are defined in shared.h and may be used in custom game DLLs. */
+
+// File searching
+static char findbase[MAX_OSPATH];
+static char findpath[MAX_OSPATH];
+static HANDLE findhandle;
+
+char *
+Sys_FindFirst(char *path, unsigned musthave, unsigned canthave)
+{
+	if (findhandle)
+	{
+		Sys_Error("Sys_BeginFind without close");
+	}
+
+	COM_FilePath(path, findbase);
+
+	WCHAR wpath[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
+
+	WIN32_FIND_DATAW findinfo;
+	findhandle = FindFirstFileW(wpath, &findinfo);
+
+	if (findhandle == INVALID_HANDLE_VALUE)
+	{
+		return NULL;
+	}
+
+	CHAR cFileName[MAX_OSPATH];
+	WideCharToMultiByte(CP_UTF8, 0, findinfo.cFileName, -1, cFileName, MAX_OSPATH, NULL, NULL);
+
+	Com_sprintf(findpath, sizeof(findpath), "%s/%s", findbase, cFileName);
+	return findpath;
+}
+
+char *
+Sys_FindNext(unsigned musthave, unsigned canthave)
+{
+	WIN32_FIND_DATAW findinfo;
+
+	if (findhandle == INVALID_HANDLE_VALUE)
+	{
+		return NULL;
+	}
+
+	if (!FindNextFileW(findhandle, &findinfo))
+	{
+		return NULL;
+	}
+
+	CHAR cFileName[MAX_OSPATH];
+	WideCharToMultiByte(CP_UTF8, 0, findinfo.cFileName, -1, cFileName, MAX_OSPATH, NULL, NULL);
+
+	Com_sprintf(findpath, sizeof(findpath), "%s/%s", findbase, cFileName);
+	return findpath;
+}
+
+void
+Sys_FindClose(void)
+{
+	if (findhandle != INVALID_HANDLE_VALUE)
+	{
+		FindClose(findhandle);
+	}
+
+	findhandle = 0;
 }
 
 /* ================================================================ */
@@ -325,6 +404,7 @@ Sys_GetGameAPI(void *parms)
 {
 	void *(*GetGameAPI)(void *);
 	char name[MAX_OSPATH];
+	WCHAR wname[MAX_OSPATH];
 	char *path = NULL;
 
 	if (game_library)
@@ -346,21 +426,23 @@ Sys_GetGameAPI(void *parms)
 
 		/* Try game.dll */
 		Com_sprintf(name, sizeof(name), "%s/%s", path, "game.dll");
-		game_library = LoadLibrary(name);
+		MultiByteToWideChar(CP_UTF8, 0, name, -1, wname, MAX_OSPATH);
+		game_library = LoadLibraryW(wname);
 
 		if (game_library)
 		{
-			Com_DPrintf("LoadLibrary (%s)\n", name);
+			Com_DPrintf("Loading library: %s\n", name);
 			break;
 		}
 
 		/* Try gamex86.dll as fallback */
  		Com_sprintf(name, sizeof(name), "%s/%s", path, "gamex86.dll");
-		game_library = LoadLibrary(name);
+		MultiByteToWideChar(CP_UTF8, 0, name, -1, wname, MAX_OSPATH);
+		game_library = LoadLibraryW(wname);
 
 		if (game_library)
 		{
-			Com_DPrintf("LoadLibrary (%s)\n", name);
+			Com_DPrintf("Loading library: %s\n", name);
 			break;
 		}
 	}
@@ -379,201 +461,50 @@ Sys_GetGameAPI(void *parms)
 /* ======================================================================= */
 
 void
-ParseCommandLine(LPSTR lpCmdLine)
+Sys_Mkdir(const char *path)
 {
-	argc = 1;
-	argv[0] = "exe";
-
-	while (*lpCmdLine && (argc < MAX_NUM_ARGVS))
+	if (!Sys_IsDir(path))
 	{
-		while (*lpCmdLine && ((*lpCmdLine <= 32) || (*lpCmdLine > 126)))
+		WCHAR wpath[MAX_OSPATH] = {0};
+		MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
+
+		if (CreateDirectoryW(wpath, NULL) == 0)
 		{
-			lpCmdLine++;
-		}
-
-		if (*lpCmdLine)
-		{
-			argv[argc] = lpCmdLine;
-			argc++;
-
-			while (*lpCmdLine && ((*lpCmdLine > 32) && (*lpCmdLine <= 126)))
-			{
-				lpCmdLine++;
-			}
-
-			if (*lpCmdLine)
-			{
-				*lpCmdLine = 0;
-				lpCmdLine++;
-			}
+			Com_Error(ERR_FATAL, "Couldn't create dir %s\n", path);
 		}
 	}
 }
 
-/* ======================================================================= */
-
-int
-Sys_Milliseconds(void)
+qboolean
+Sys_IsDir(const char *path)
 {
-	static int base;
-	static qboolean initialized = false;
+	WCHAR wpath[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
 
-	if (!initialized)
-	{   /* let base retain 16 bits of effectively random data */
-		base = timeGetTime() & 0xffff0000;
-		initialized = true;
+	DWORD fileAttributes = GetFileAttributesW(wpath);
+	if (fileAttributes == INVALID_FILE_ATTRIBUTES)
+	{
+		return false;
 	}
 
-	curtime = timeGetTime() - base;
-
-	return curtime;
+	return (fileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
 }
 
-void
-Sys_Sleep(int msec)
+qboolean
+Sys_IsFile(const char *path)
 {
-	Sleep(msec);
-}
+	WCHAR wpath[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
 
-/* ======================================================================= */
-
-static qboolean
-CompareAttributes(unsigned found, unsigned musthave, unsigned canthave)
-{
-	if ((found & _A_RDONLY) && (canthave & SFF_RDONLY))
+	DWORD fileAttributes = GetFileAttributesW(wpath);
+	if (fileAttributes == INVALID_FILE_ATTRIBUTES)
 	{
 		return false;
 	}
 
-	if ((found & _A_HIDDEN) && (canthave & SFF_HIDDEN))
-	{
-		return false;
-	}
-
-	if ((found & _A_SYSTEM) && (canthave & SFF_SYSTEM))
-	{
-		return false;
-	}
-
-	if ((found & _A_SUBDIR) && (canthave & SFF_SUBDIR))
-	{
-		return false;
-	}
-
-	if ((found & _A_ARCH) && (canthave & SFF_ARCH))
-	{
-		return false;
-	}
-
-	if ((musthave & SFF_RDONLY) && !(found & _A_RDONLY))
-	{
-		return false;
-	}
-
-	if ((musthave & SFF_HIDDEN) && !(found & _A_HIDDEN))
-	{
-		return false;
-	}
-
-	if ((musthave & SFF_SYSTEM) && !(found & _A_SYSTEM))
-	{
-		return false;
-	}
-
-	if ((musthave & SFF_SUBDIR) && !(found & _A_SUBDIR))
-	{
-		return false;
-	}
-
-	if ((musthave & SFF_ARCH) && !(found & _A_ARCH))
-	{
-		return false;
-	}
-
-	return true;
-}
-
-char *
-Sys_FindFirst(char *path, unsigned musthave, unsigned canthave)
-{
-	struct _finddata_t findinfo;
-
-	if (findhandle)
-	{
-		Sys_Error("Sys_BeginFind without close");
-	}
-
-	findhandle = 0;
-
-	COM_FilePath(path, findbase);
-	findhandle = _findfirst(path, &findinfo);
-
-	if (findhandle == -1)
-	{
-		return NULL;
-	}
-
-	if (!CompareAttributes(findinfo.attrib, musthave, canthave))
-	{
-		return NULL;
-	}
-
-	Com_sprintf(findpath, sizeof(findpath), "%s/%s", findbase, findinfo.name);
-	return findpath;
-}
-
-char *
-Sys_FindNext(unsigned musthave, unsigned canthave)
-{
-	struct _finddata_t findinfo;
-
-	if (findhandle == -1)
-	{
-		return NULL;
-	}
-
-	if (_findnext(findhandle, &findinfo) == -1)
-	{
-		return NULL;
-	}
-
-	if (!CompareAttributes(findinfo.attrib, musthave, canthave))
-	{
-		return NULL;
-	}
-
-	Com_sprintf(findpath, sizeof(findpath), "%s/%s", findbase, findinfo.name);
-	return findpath;
-}
-
-void
-Sys_FindClose(void)
-{
-	if (findhandle != -1)
-	{
-		_findclose(findhandle);
-	}
-
-	findhandle = 0;
-}
-
-void
-Sys_Mkdir(char *path)
-{
-	_mkdir(path);
-}
-
-char *
-Sys_GetCurrentDirectory(void)
-{
-	static char dir[MAX_OSPATH];
-
-	if (!_getcwd(dir, sizeof(dir)))
-	{
-		Sys_Error("Couldn't get current working directory");
-	}
-
-	return dir;
+	// I guess the assumption that if it's not a file or device
+	// then it's a directory is good enough for us?
+	return (fileAttributes & (FILE_ATTRIBUTE_DIRECTORY|FILE_ATTRIBUTE_DEVICE)) == 0;
 }
 
 char *
@@ -582,40 +513,13 @@ Sys_GetHomeDir(void)
 	char *cur;
 	char *old;
 	char profile[MAX_PATH];
-	int len;
 	static char gdir[MAX_OSPATH];
-	WCHAR sprofile[MAX_PATH];
 	WCHAR uprofile[MAX_PATH];
 
-	/* The following lines implement a horrible
-	   hack to connect the UTF-16 WinAPI to the
-	   ASCII Quake II. While this should work in
-	   most cases, it'll fail if the "Windows to
-	   DOS filename translation" is switched off.
-	   In that case the function will return NULL
-	   and no homedir is used. */
-
 	/* Get the path to "My Documents" directory */
 	SHGetFolderPathW(NULL, CSIDL_PERSONAL, NULL, 0, uprofile);
 
-	/* Create a UTF-16 DOS path */
-    len = GetShortPathNameW(uprofile, sprofile, sizeof(sprofile));
-
-	if (len == 0)
-	{
-		return NULL;
-	}
-
-	/* Since the DOS path contains no UTF-16 characters, just convert it to ASCII */
-	WideCharToMultiByte(CP_ACP, 0, sprofile, -1, profile, sizeof(profile), NULL, NULL);
-
-	if (len == 0)
-	{
-		return NULL;
-	}
-
-	/* Check if path is too long */
-    if ((len + strlen(CFGDIR) + 3) >= 256)
+	if (WideCharToMultiByte(CP_UTF8, 0, uprofile, -1, profile, sizeof(profile), NULL, NULL) == 0)
 	{
 		return NULL;
 	}
@@ -638,56 +542,235 @@ Sys_GetHomeDir(void)
 		}
 	}
 
-	snprintf(gdir, sizeof(gdir), "%s/%s/", profile, CFGDIR);
+	snprintf(gdir, sizeof(gdir), "%s/%s/", profile, cfgdir);
+	Sys_Mkdir(gdir);
 
 	return gdir;
 }
 
 void
-Sys_RedirectStdout(void)
+Sys_Remove(const char *path)
 {
-	char *cur;
-	char *home;
-	char *old;
-	char path_stdout[MAX_OSPATH];
-	char path_stderr[MAX_OSPATH];
+	WCHAR wpath[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
 
-	if (portable->value)
+	_wremove(wpath);
+}
+
+int
+Sys_Rename(const char *from, const char *to)
+{
+	WCHAR wfrom[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, from, -1, wfrom, MAX_OSPATH);
+
+	WCHAR wto[MAX_OSPATH] = {0};
+	MultiByteToWideChar(CP_UTF8, 0, to, -1, wto, MAX_OSPATH);
+
+	return _wrename(wfrom, wto);
+}
+
+void
+Sys_RemoveDir(const char *path)
+{
+	WCHAR wpath[MAX_OSPATH] = {0};
+	WCHAR wpathwithwildcard[MAX_OSPATH] = {0};
+	WCHAR wpathwithfilename[MAX_OSPATH] = {0};
+	WIN32_FIND_DATAW fd;
+	
+	if (MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH) >= MAX_QPATH)
 	{
+		/* This is hopefully never reached, because in a good
+		   world path has a maximum length of MAX_QPATH. Since
+		   we can't use wcscat_s() below because it would break
+		   compat with Win XP, nevertheless do this check. */
 		return;
 	}
 
-	home = Sys_GetHomeDir();
-
-	if (home == NULL)
+	wcsncpy(wpathwithwildcard, wpath, MAX_OSPATH);
+	wcscat(wpathwithwildcard, L"\\*.*");
+	
+	HANDLE hFind = FindFirstFileW(wpathwithwildcard, &fd);
+	if (hFind != INVALID_HANDLE_VALUE)
 	{
-		return;
-	}
-
-	cur = old = home;
-
-	while (cur != NULL)
-	{
-		if ((cur - old) > 1)
+		do
 		{
-			*cur = '\0';
-			Sys_Mkdir(home);
-			*cur = '/';
-		}
+			if (wcslen(wpath) + wcslen(fd.cFileName) >= MAX_QPATH)
+			{
+				// Same as above.
+				continue;
+			}
 
-		old = cur;
-		cur = strchr(old + 1, '/');
+			wmemset(wpathwithfilename, 0, MAX_OSPATH);
+			wcscat(wpathwithfilename, wpath);
+			wcscat(wpathwithfilename, fd.cFileName);
+			
+			DeleteFileW(wpathwithfilename);
+		}
+		while (FindNextFileW(hFind, &fd));
+		FindClose(hFind);
+	}
+	
+	RemoveDirectoryW(wpath);
+}
+
+qboolean
+Sys_Realpath(const char *in, char *out, size_t size)
+{
+	WCHAR win[MAX_OSPATH] = {0};
+	WCHAR wconverted[MAX_OSPATH] = {0};
+
+	MultiByteToWideChar(CP_UTF8, 0, in, -1, win, sizeof(win)/sizeof(win[0]));
+
+	if (_wfullpath(wconverted, win, size) == NULL)
+	{
+		Com_Printf("Couldn't get realpath for %s\n", in);
+		return false;
 	}
 
-	snprintf(path_stdout, sizeof(path_stdout), "%s/%s", home, "stdout.txt");
-	snprintf(path_stderr, sizeof(path_stderr), "%s/%s", home, "stderr.txt");
-
-	freopen(path_stdout, "w", stdout);
-	freopen(path_stderr, "w", stderr);
+	WideCharToMultiByte(CP_UTF8, 0, wconverted, -1, out, size, NULL, NULL);
+	return true;
 }
 
 /* ======================================================================= */
 
+void *
+Sys_GetProcAddress(void *handle, const char *sym)
+{
+	return GetProcAddress(handle, sym);
+}
+
+void
+Sys_FreeLibrary(void *handle)
+{
+	if (!handle)
+	{
+		return;
+	}
+
+	if (!FreeLibrary(handle))
+	{
+		Com_Error(ERR_FATAL, "FreeLibrary failed on %p", handle);
+	}
+}
+
+void *
+Sys_LoadLibrary(const char *path, const char *sym, void **handle)
+{
+	HMODULE module;
+	void *entry;
+	WCHAR wpath[MAX_OSPATH];
+
+	*handle = NULL;
+
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, MAX_OSPATH);
+	module = LoadLibraryW(wpath);
+
+	if (!module)
+	{
+		Com_Printf("%s failed: LoadLibrary returned %lu on %s\n",
+		           __func__, GetLastError(), path);
+		return NULL;
+	}
+
+	if (sym)
+	{
+		entry = GetProcAddress(module, sym);
+
+		if (!entry)
+		{
+			Com_Printf("%s failed: GetProcAddress returned %lu on %s\n",
+			           __func__, GetLastError(), path);
+			FreeLibrary(module);
+			return NULL;
+		}
+	}
+	else
+	{
+		entry = NULL;
+	}
+
+	*handle = module;
+
+	Com_DPrintf("%s succeeded: %s\n", __func__, path);
+
+	return entry;
+}
+
+/* ======================================================================= */
+
+void
+Sys_GetWorkDir(char *buffer, size_t len)
+{
+	WCHAR wbuffer[MAX_OSPATH];
+
+	if (GetCurrentDirectoryW(sizeof(wbuffer)/sizeof(wbuffer[0]), wbuffer) != 0)
+	{
+		WideCharToMultiByte(CP_UTF8, 0, wbuffer, -1, buffer, len, NULL, NULL);
+		return;
+	}
+
+	buffer[0] = '\0';
+}
+
+qboolean
+Sys_SetWorkDir(char *path)
+{
+	WCHAR wpath[MAX_OSPATH];
+
+	MultiByteToWideChar(CP_UTF8, 0, path, -1, wpath, sizeof(wpath)/sizeof(wpath[0]));
+
+	if (SetCurrentDirectoryW(wpath) != 0)
+	{
+		return true;
+	}
+
+	return false;
+}
+
+/* ======================================================================= */
+
+// This one is Windows specific.
+
+void
+Sys_RedirectStdout(void)
+{
+	char dir[MAX_OSPATH];
+	char path_stdout[MAX_OSPATH];
+	char path_stderr[MAX_OSPATH];
+	WCHAR wpath_stdout[MAX_OSPATH];
+	WCHAR wpath_stderr[MAX_OSPATH];
+	const char *tmp;
+
+	if (is_portable) {
+		tmp = Sys_GetBinaryDir();
+		Q_strlcpy(dir, tmp, sizeof(dir));
+	} else {
+		tmp = Sys_GetHomeDir();
+		Q_strlcpy(dir, tmp, sizeof(dir));
+	}
+
+	if (dir[0] == '\0')
+	{
+		return;
+	}
+
+	snprintf(path_stdout, sizeof(path_stdout), "%s/%s", dir, "stdout.txt");
+	snprintf(path_stderr, sizeof(path_stderr), "%s/%s", dir, "stderr.txt");
+
+	MultiByteToWideChar(CP_UTF8, 0, path_stdout, -1, wpath_stdout, sizeof(wpath_stdout)/sizeof(wpath_stdout[0]));
+	MultiByteToWideChar(CP_UTF8, 0, path_stderr, -1, wpath_stderr, sizeof(wpath_stderr)/sizeof( wpath_stderr[0] ) );
+
+	_wfreopen(wpath_stdout, L"w", stdout);
+	_wfreopen(wpath_stderr, L"w", stderr);
+
+	setvbuf(stdout, (char *)NULL, _IOLBF, BUFSIZ);
+	setvbuf(stderr, (char *)NULL, _IOLBF, BUFSIZ);
+}
+
+/* ======================================================================= */
+
+// This one is windows specific.
+
 typedef enum YQ2_PROCESS_DPI_AWARENESS {
 	YQ2_PROCESS_DPI_UNAWARE = 0,
 	YQ2_PROCESS_SYSTEM_DPI_AWARE = 1,
@@ -729,174 +812,3 @@ Sys_SetHighDPIMode(void)
 		SetProcessDPIAware();
 	}
 }
-
-/* ======================================================================= */
-
-/*
- * Windows main function. Containts the
- * initialization code and the main loop
- */
-int WINAPI
-WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
-		LPSTR lpCmdLine, int nCmdShow)
-{
-	MSG msg;
-	int time, oldtime, newtime;
-
-	/* Previous instances do not exist in Win32 */
-	if (hPrevInstance)
-	{
-		return 0;
-	}
-
-	/* Make the current instance global */
-	global_hInstance = hInstance;
-
-	/* Force DPI awareness */
-	Sys_SetHighDPIMode();
-
-	/* FIXME: No one can see this! */
-	printf("Yamagi Quake II v%s\n", YQ2VERSION);
-	printf("=====================\n\n");
-
-#ifndef DEDICATED_ONLY
-	printf("Client build options:\n");
-#ifdef SDL2
-	printf(" + SDL2\n");
-#else
-	printf(" - SDL2 (using 1.2)\n");
-#endif
-#ifdef CDA
-	printf(" + CD audio\n");
-#else
-	printf(" - CD audio\n");
-#endif
-#ifdef OGG
-	printf(" + OGG/Vorbis\n");
-#else
-	printf(" - OGG/Vorbis\n");
-#endif
-#ifdef USE_OPENAL
-	printf(" + OpenAL audio\n");
-#else
-	printf(" - OpenAL audio\n");
-#endif
-#ifdef ZIP
-	printf(" + Zip file support\n");
-#else
-	printf(" - Zip file support\n");
-#endif
-#endif
-
-	printf("Platform: %s\n", YQ2OSTYPE);
-	printf("Architecture: %s\n", YQ2ARCH);
-
-	/* Seed PRNG */
-	randk_seed();
-
-	/* Parse the command line arguments */
-	ParseCommandLine(lpCmdLine);
-
-	/* Call the initialization code */
-	Qcommon_Init(argc, argv);
-
-	/* Save our time */
-	oldtime = Sys_Milliseconds();
-
-	/* The legendary main loop */
-	while (1)
-	{
-		/* If at a full screen console, don't update unless needed */
-		if (Minimized || (dedicated && dedicated->value))
-		{
-			Sleep(1);
-		}
-
-		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
-		{
-			if (!GetMessage(&msg, NULL, 0, 0))
-			{
-				Com_Quit();
-			}
-
-			sys_msg_time = msg.time;
-			TranslateMessage(&msg);
-			DispatchMessage(&msg);
-		}
-
-		do
-		{
-			newtime = Sys_Milliseconds();
-			time = newtime - oldtime;
-		}
-		while (time < 1);
-
-		Qcommon_Frame(time);
-		oldtime = newtime;
-	}
-
-	/* never gets here */
-	return TRUE;
-}
-
-void
-Sys_FreeLibrary(void *handle)
-{
-	if (!handle)
-	{
-		return;
-	}
-
-	if (!FreeLibrary(handle))
-	{
-		Com_Error(ERR_FATAL, "FreeLibrary failed on %p", handle);
-	}
-}
-
-void *
-Sys_LoadLibrary(const char *path, const char *sym, void **handle)
-{
-	HMODULE module;
-	void *entry;
-
-	*handle = NULL;
-
-	module = LoadLibraryA(path);
-
-	if (!module)
-	{
-		Com_Printf("%s failed: LoadLibrary returned %lu on %s\n",
-				__func__, GetLastError(), path);
-		return NULL;
-	}
-
-	if (sym)
-	{
-		entry = GetProcAddress(module, sym);
-
-		if (!entry)
-		{
-			Com_Printf("%s failed: GetProcAddress returned %lu on %s\n",
-					__func__, GetLastError(), path);
-			FreeLibrary(module);
-			return NULL;
-		}
-	}
-	else
-	{
-		entry = NULL;
-	}
-
-	*handle = module;
-
-	Com_DPrintf("%s succeeded: %s\n", __func__, path);
-
-	return entry;
-}
-
-void *
-Sys_GetProcAddress(void *handle, const char *sym)
-{
-	return GetProcAddress(handle, sym);
-}
-
