@@ -29,72 +29,6 @@
 
 extern struct model_s *cl_mod_powerscreen;
 
-struct model_s *
-S_RegisterSexedModel(entity_state_t *ent, char *base)
-{
-	int n;
-	char *p;
-	struct model_s *md2;
-	char model[MAX_QPATH];
-	char buffer[MAX_QPATH];
-
-	/* determine what model the client is using */
-	model[0] = 0;
-	n = CS_PLAYERSKINS + ent->number - 1;
-
-	if (cl.configstrings[n][0])
-	{
-		p = strchr(cl.configstrings[n], '\\');
-
-		if (p)
-		{
-			p += 1;
-			strcpy(model, p);
-			p = strchr(model, '/');
-
-			if (p)
-			{
-				*p = 0;
-			}
-		}
-	}
-
-	/* if we can't figure it out, they're male */
-	if (!model[0])
-	{
-		strcpy(model, "male");
-	}
-
-	Com_sprintf(buffer, sizeof(buffer), "players/%s/%s", model, base + 1);
-	md2 = R_RegisterModel(buffer);
-
-	if (!md2)
-	{
-		/* not found, try default weapon model */
-		Com_sprintf(buffer, sizeof(buffer), "players/%s/weapon.md2", model);
-		md2 = R_RegisterModel(buffer);
-
-		if (!md2)
-		{
-			/* no, revert to the male model */
-			Com_sprintf(buffer, sizeof(buffer), "players/%s/%s",
-					"male", base + 1);
-			md2 = R_RegisterModel(buffer);
-
-			if (!md2)
-			{
-				/* last try, default male weapon.md2 */
-				Com_sprintf(buffer, sizeof(buffer), "players/male/weapon.md2");
-				md2 = R_RegisterModel(buffer);
-			}
-		}
-	}
-
-	return md2;
-}
-
-extern int Developer_searchpath(int who);
-
 void
 CL_AddPacketEntities(frame_t *frame)
 {
@@ -108,6 +42,9 @@ CL_AddPacketEntities(frame_t *frame)
 	clientinfo_t *ci;
 	unsigned int effects, renderfx;
 
+	/* To distinguish baseq2, xatrix and rogue. */
+	cvar_t *game = Cvar_Get("game",  "", CVAR_LATCH | CVAR_SERVERINFO);
+
 	/* bonus items rotate at a fixed rate */
 	autorotate = anglemod(cl.time * 0.1f);
 
@@ -377,7 +314,7 @@ CL_AddPacketEntities(frame_t *frame)
 			   something special */
 			if (renderfx & RF_SHELL_HALF_DAM)
 			{
-				if (Developer_searchpath(2) == 2)
+				if (strcmp(game->string, "rogue") == 0)
 				{
 					/* ditch the half damage shell if any of red, blue, or double are on */
 					if (renderfx & (RF_SHELL_RED | RF_SHELL_BLUE | RF_SHELL_DOUBLE))
@@ -389,7 +326,7 @@ CL_AddPacketEntities(frame_t *frame)
 
 			if (renderfx & RF_SHELL_DOUBLE)
 			{
-				if (Developer_searchpath(2) == 2)
+				if (strcmp(game->string, "rogue") == 0)
 				{
 					/* lose the yellow shell if we have a red, blue, or green shell */
 					if (renderfx & (RF_SHELL_RED | RF_SHELL_BLUE | RF_SHELL_GREEN))
@@ -508,7 +445,15 @@ CL_AddPacketEntities(frame_t *frame)
 			if (effects & EF_ROCKET)
 			{
 				CL_RocketTrail(cent->lerp_origin, ent.origin, cent);
-				V_AddLight(ent.origin, 200, 1, 0.25f, 0);
+
+				if (cl_r1q2_lightstyle->value)
+				{
+					V_AddLight(ent.origin, 200, 1, 0.23f, 0);
+				}
+				else
+				{
+					V_AddLight(ent.origin, 200, 1, 1, 0);
+				}
 			}
 
 			/* Do not reorder EF_BLASTER and EF_HYPERBLASTER.
@@ -533,7 +478,6 @@ CL_AddPacketEntities(frame_t *frame)
 				{
 					V_AddLight(ent.origin, 200, 0, 1, 0);
 				}
-
 				else
 				{
 					V_AddLight(ent.origin, 200, 1, 1, 0);
@@ -820,10 +764,13 @@ CL_CalcViewValues(void)
 		}
 	}
 
-	for (i = 0; i < 3; i++)
+	if (cl_kickangles->value)
 	{
-		cl.refdef.viewangles[i] += LerpAngle(ops->kick_angles[i],
-				ps->kick_angles[i], lerp);
+		for (i = 0; i < 3; i++)
+		{
+			cl.refdef.viewangles[i] += LerpAngle(ops->kick_angles[i],
+					ps->kick_angles[i], lerp);
+		}
 	}
 
 	AngleVectors(cl.refdef.viewangles, cl.v_forward, cl.v_right, cl.v_up);
