@@ -26,7 +26,7 @@
  */
 
 #include "header/client.h"
-#include "../backends/generic/header/input.h"
+#include "input/header/input.h"
 
 void CL_ForwardToServer_f(void);
 void CL_Changing_f(void);
@@ -35,8 +35,6 @@ void CL_Connect_f(void);
 void CL_Rcon_f(void);
 void CL_CheckForResend(void);
 
-cvar_t *freelook;
-
 cvar_t *rcon_client_password;
 cvar_t *rcon_address;
 
@@ -44,32 +42,26 @@ cvar_t *cl_noskins;
 cvar_t *cl_footsteps;
 cvar_t *cl_timeout;
 cvar_t *cl_predict;
-cvar_t *cl_maxfps;
-cvar_t *cl_drawfps;
+cvar_t *cl_showfps;
 cvar_t *cl_gun;
 cvar_t *cl_add_particles;
 cvar_t *cl_add_lights;
 cvar_t *cl_add_entities;
 cvar_t *cl_add_blend;
-cvar_t *cl_async;
+cvar_t *cl_kickangles;
 
 cvar_t *cl_shownet;
 cvar_t *cl_showmiss;
 cvar_t *cl_showclamp;
 
 cvar_t *cl_paused;
-cvar_t *cl_timedemo;
-
-cvar_t *lookspring;
-cvar_t *lookstrafe;
-cvar_t *sensitivity;
-
-cvar_t *m_pitch;
-cvar_t *m_yaw;
-cvar_t *m_forward;
-cvar_t *m_side;
+cvar_t *cl_loadpaused;
+cvar_t *cl_audiopaused;
+cvar_t *cl_unpaused_scvis;
 
 cvar_t *cl_lightlevel;
+cvar_t *cl_r1q2_lightstyle;
+cvar_t *cl_limitsparksounds;
 
 /* userinfo */
 cvar_t *name;
@@ -78,16 +70,13 @@ cvar_t *rate;
 cvar_t *fov;
 cvar_t *horplus;
 cvar_t *windowed_mouse;
-cvar_t *msg;
 cvar_t *hand;
 cvar_t *gender;
 cvar_t *gender_auto;
 
-cvar_t *gl_maxfps;
-
-cvar_t	*gl_stereo;
-cvar_t	*gl_stereo_separation;
-cvar_t	*gl_stereo_convergence;
+cvar_t	*gl1_stereo;
+cvar_t	*gl1_stereo_separation;
+cvar_t	*gl1_stereo_convergence;
 
 cvar_t *cl_vwep;
 
@@ -98,6 +87,16 @@ centity_t cl_entities[MAX_EDICTS];
 
 entity_state_t cl_parse_entities[MAX_PARSE_ENTITIES];
 
+/*Evil hack against too many power screen and power
+  shield impact sounds. For example if the player
+  fires his shotgun onto a Brain. */
+int num_power_sounds;
+
+/* Used to communicate if we entered paused mode
+   during server connect or if we were already in
+   it. */
+qboolean paused_at_load;
+
 extern cvar_t *allow_download;
 extern cvar_t *allow_download_players;
 extern cvar_t *allow_download_models;
@@ -179,7 +178,7 @@ CL_Record_f(void)
 
 	Com_Printf("recording to %s.\n", name);
 	FS_CreatePath(name);
-	cls.demofile = fopen(name, "wb");
+	cls.demofile = Q_fopen(name, "wb");
 
 	if (!cls.demofile)
 	{
@@ -271,13 +270,13 @@ CL_Setenv_f(void)
 		char buffer[1000];
 		int i;
 
-		strcpy(buffer, Cmd_Argv(1));
-		strcat(buffer, "=");
+		Q_strlcpy(buffer, Cmd_Argv(1), sizeof(buffer));
+		Q_strlcat(buffer, "=", sizeof(buffer));
 
 		for (i = 2; i < argc; i++)
 		{
-			strcat(buffer, Cmd_Argv(i));
-			strcat(buffer, " ");
+			Q_strlcat(buffer, Cmd_Argv(i), sizeof(buffer));
+			Q_strlcat(buffer, " ", sizeof(buffer));
 		}
 
 		putenv(buffer);
@@ -310,6 +309,17 @@ CL_Pause_f(void)
 	}
 
 	Cvar_SetValue("paused", !cl_paused->value);
+
+	if (Cvar_VariableValue("ogg_pausewithgame") == 1 &&
+	    OGG_Status() == PAUSE && cl_paused->value == 0)			/* play music */
+	{
+	    Cbuf_AddText("ogg toggle\n");
+	}
+	else if (Cvar_VariableValue("ogg_pausewithgame") == 1 &&
+	    OGG_Status() == PLAY && cl_paused->value == 1)			/* pause music */
+	{
+	    Cbuf_AddText("ogg toggle\n");
+	}
 }
 
 void
@@ -366,7 +376,7 @@ CL_Skins_f(void)
 
 		SCR_UpdateScreen();
 
-		Sys_SendKeyEvents();  /* pump message loop */
+		IN_Update();  /* pump message loop */
 
 		CL_ParseClientinfo(i);
 	}
@@ -426,18 +436,30 @@ CL_Userinfo_f(void)
 void
 CL_Snd_Restart_f(void)
 {
+	OGG_SaveState();
+
 	S_Shutdown();
 	S_Init();
+
 	CL_RegisterSounds();
+
+	OGG_InitTrackList();
+	OGG_RecoverState();
 }
 
 int precache_check;
 int precache_spawncount;
 int precache_tex;
 int precache_model_skin;
-
 byte *precache_model;
 
+void CL_ResetPrecacheCheck (void)
+{
+	precache_check = CS_MODELS;
+	precache_model = 0;
+	precache_model_skin = 0;
+}
+
 /*
  * The server will send this command right
  * before allowing the client into the server
@@ -465,6 +487,11 @@ CL_Precache_f(void)
 	CL_RequestNextDownload();
 }
 
+void CL_CurrentMap_f(void)
+{
+	Com_Printf("%s\n", cl.configstrings[CS_MODELS + 1]);
+}
+
 void
 CL_InitLocal(void)
 {
@@ -480,13 +507,12 @@ CL_InitLocal(void)
 	cl_add_lights = Cvar_Get("cl_lights", "1", 0);
 	cl_add_particles = Cvar_Get("cl_particles", "1", 0);
 	cl_add_entities = Cvar_Get("cl_entities", "1", 0);
+	cl_kickangles = Cvar_Get("cl_kickangles", "1", 0);
 	cl_gun = Cvar_Get("cl_gun", "2", CVAR_ARCHIVE);
 	cl_footsteps = Cvar_Get("cl_footsteps", "1", 0);
 	cl_noskins = Cvar_Get("cl_noskins", "0", 0);
 	cl_predict = Cvar_Get("cl_predict", "1", 0);
-	cl_maxfps = Cvar_Get("cl_maxfps", "60", CVAR_ARCHIVE);
-	cl_drawfps = Cvar_Get("cl_drawfps", "0", CVAR_ARCHIVE);
-	cl_async = Cvar_Get("cl_async", "1", CVAR_ARCHIVE);
+	cl_showfps = Cvar_Get("cl_showfps", "0", CVAR_ARCHIVE);
 
 	cl_upspeed = Cvar_Get("cl_upspeed", "200", 0);
 	cl_forwardspeed = Cvar_Get("cl_forwardspeed", "200", 0);
@@ -496,39 +522,31 @@ CL_InitLocal(void)
 	cl_anglespeedkey = Cvar_Get("cl_anglespeedkey", "1.5", 0);
 
 	cl_run = Cvar_Get("cl_run", "0", CVAR_ARCHIVE);
-	freelook = Cvar_Get("freelook", "1", CVAR_ARCHIVE);
-	lookspring = Cvar_Get("lookspring", "0", CVAR_ARCHIVE);
-	lookstrafe = Cvar_Get("lookstrafe", "0", CVAR_ARCHIVE);
-	sensitivity = Cvar_Get("sensitivity", "3", CVAR_ARCHIVE);
-
-	m_pitch = Cvar_Get("m_pitch", "0.022", CVAR_ARCHIVE);
-	m_yaw = Cvar_Get("m_yaw", "0.022", 0);
-	m_forward = Cvar_Get("m_forward", "1", 0);
-	m_side = Cvar_Get("m_side", "1", 0);
 
 	cl_shownet = Cvar_Get("cl_shownet", "0", 0);
 	cl_showmiss = Cvar_Get("cl_showmiss", "0", 0);
 	cl_showclamp = Cvar_Get("showclamp", "0", 0);
 	cl_timeout = Cvar_Get("cl_timeout", "120", 0);
 	cl_paused = Cvar_Get("paused", "0", 0);
-	cl_timedemo = Cvar_Get("timedemo", "0", 0);
+	cl_loadpaused = Cvar_Get("cl_loadpaused", "1", CVAR_ARCHIVE);
+	cl_audiopaused = Cvar_Get("cl_audiopaused", "1", CVAR_ARCHIVE);
+	cl_unpaused_scvis = Cvar_Get("cl_unpaused_scvis", "1", CVAR_ARCHIVE);
 
-	gl_maxfps = Cvar_Get("gl_maxfps", "95", CVAR_ARCHIVE);
-
-	gl_stereo = Cvar_Get( "gl_stereo", "0", CVAR_ARCHIVE );
-	gl_stereo_separation = Cvar_Get( "gl_stereo_separation", "1", CVAR_ARCHIVE );
-	gl_stereo_convergence = Cvar_Get( "gl_stereo_convergence", "1.4", CVAR_ARCHIVE );
+	gl1_stereo = Cvar_Get( "gl1_stereo", "0", CVAR_ARCHIVE );
+	gl1_stereo_separation = Cvar_Get( "gl1_stereo_separation", "1", CVAR_ARCHIVE );
+	gl1_stereo_convergence = Cvar_Get( "gl1_stereo_convergence", "1.4", CVAR_ARCHIVE );
 
 	rcon_client_password = Cvar_Get("rcon_password", "", 0);
 	rcon_address = Cvar_Get("rcon_address", "", 0);
 
-	cl_lightlevel = Cvar_Get("gl_lightlevel", "0", 0);
+	cl_lightlevel = Cvar_Get("r_lightlevel", "0", 0);
+	cl_r1q2_lightstyle = Cvar_Get("cl_r1q2_lightstyle", "1", CVAR_ARCHIVE);
+	cl_limitsparksounds = Cvar_Get("cl_limitsparksounds", "0", CVAR_ARCHIVE);
 
 	/* userinfo */
 	name = Cvar_Get("name", "unnamed", CVAR_USERINFO | CVAR_ARCHIVE);
 	skin = Cvar_Get("skin", "male/grunt", CVAR_USERINFO | CVAR_ARCHIVE);
 	rate = Cvar_Get("rate", "8000", CVAR_USERINFO | CVAR_ARCHIVE);
-	msg = Cvar_Get("msg", "1", CVAR_USERINFO | CVAR_ARCHIVE);
 	hand = Cvar_Get("hand", "0", CVAR_USERINFO | CVAR_ARCHIVE);
 	fov = Cvar_Get("fov", "90", CVAR_USERINFO | CVAR_ARCHIVE);
 	horplus = Cvar_Get("horplus", "1", CVAR_ARCHIVE);
@@ -543,6 +561,16 @@ CL_InitLocal(void)
 
 	cl_vwep = Cvar_Get("cl_vwep", "1", CVAR_ARCHIVE);
 
+#ifdef USE_CURL
+	cl_http_proxy = Cvar_Get("cl_http_proxy", "", 0);
+	cl_http_filelists = Cvar_Get("cl_http_filelists", "1", 0);
+	cl_http_downloads = Cvar_Get("cl_http_downloads", "1", CVAR_ARCHIVE);
+	cl_http_max_connections = Cvar_Get("cl_http_max_connections", "4", 0);
+	cl_http_show_dw_progress = Cvar_Get("cl_http_show_dw_progress", "0", 0);
+	cl_http_bw_limit_rate = Cvar_Get("cl_http_bw_limit_rate", "0", 0);
+	cl_http_bw_limit_tmout = Cvar_Get("cl_http_bw_limit_tmout", "0", 0);
+#endif
+
 	/* register our commands */
 	Cmd_AddCommand("cmd", CL_ForwardToServer_f);
 	Cmd_AddCommand("pause", CL_Pause_f);
@@ -570,6 +598,8 @@ CL_InitLocal(void)
 
 	Cmd_AddCommand("download", CL_Download_f);
 
+	Cmd_AddCommand("currentmap", CL_CurrentMap_f);
+
 	/* forward to server commands
 	 * the only thing this does is allow command completion
 	 * to work -- all unknown commands are automatically
@@ -593,6 +623,11 @@ CL_InitLocal(void)
 	Cmd_AddCommand("invdrop", NULL);
 	Cmd_AddCommand("weapnext", NULL);
 	Cmd_AddCommand("weapprev", NULL);
+	Cmd_AddCommand("listentities", NULL);
+	Cmd_AddCommand("teleport", NULL);
+	Cmd_AddCommand("spawnentity", NULL);
+	Cmd_AddCommand("spawnonstart", NULL);
+	Cmd_AddCommand("cycleweap", NULL);
 }
 
 /*
@@ -611,7 +646,7 @@ CL_WriteConfiguration(void)
 
 	Com_sprintf(path, sizeof(path), "%s/config.cfg", FS_Gamedir());
 
-	f = fopen(path, "w");
+	f = Q_fopen(path, "w");
 
 	if (!f)
 	{
@@ -639,15 +674,19 @@ typedef struct
 cheatvar_t cheatvars[] = {
 	{"timescale", "1"},
 	{"timedemo", "0"},
-	{"gl_drawworld", "1"},
+	{"r_drawworld", "1"},
 	{"cl_testlights", "0"},
-	{"gl_fullbright", "0"},
+	{"r_fullbright", "0"},
 	{"gl_drawflat", "0"},
 	{"paused", "0"},
 	{"fixedtime", "0"},
 	{"sw_draworder", "0"},
-	{"gl_lightmap", "0"},
+	{"r_lightmap", "0"},
 	{"gl_saturatelighting", "0"},
+	{"cl_kickangles", "1"},
+	{"g_footsteps", "1"},
+	{"g_machinegun_norecoil", "0"},
+	{"g_swap_speed", "1"},
 	{NULL, NULL}
 };
 
@@ -711,60 +750,24 @@ CL_UpdateWindowedMouse(void)
 	}
 }
 
-int GLimp_GetRefreshRate(void);
-qboolean R_IsVSyncActive(void);
-
 void
-CL_Frame(int msec)
+CL_Frame(int packetdelta, int renderdelta, int timedelta, qboolean packetframe, qboolean renderframe)
 {
-	int nfps;
-	int rfps;
-
 	static int lasttimecalled;
 
-	static int packetdelta = 1000;
-	static int renderdelta = 1000;
-	static int miscdelta = 1000;
-
-	qboolean packetframe = true;
-	qboolean renderframe = true;
-	qboolean miscframe = true;
-
+	// Dedicated?
 	if (dedicated->value)
 	{
 		return;
 	}
 
-	// Target render frame rate
-	if (R_IsVSyncActive())
-	{
-		rfps = GLimp_GetRefreshRate();
-
-		if (rfps > gl_maxfps->value)
-		{
-			rfps = (int)gl_maxfps->value;
-		}
-	}
-	else
-	{
-		rfps = (int)gl_maxfps->value;
-	}
-
-	// The network framerate must not be higher then the render framerate
-	nfps = (cl_maxfps->value > rfps) ? rfps : cl_maxfps->value;
-
-	// Adjust deltas
-	packetdelta += msec;
-	renderdelta += msec;
-	miscdelta += msec;
-
-	// Calculate simulation time
-	cls.nframetime = packetdelta * 0.001f;
-	cls.rframetime = renderdelta * 0.001f;
+	// Calculate simulation time.
+	cls.nframetime = packetdelta / 1000000.0f;
+	cls.rframetime = renderdelta / 1000000.0f;
 	cls.realtime = curtime;
-	cl.time += msec;
+	cl.time += timedelta / 1000;
 
-	// Don't extrapolate too far ahead
+	// Don't extrapolate too far ahead.
 	if (cls.nframetime > 0.5f)
 	{
 		cls.nframetime = 0.5f;
@@ -775,80 +778,38 @@ CL_Frame(int msec)
 		cls.rframetime = 0.5f;
 	}
 
-	/* if in the debugger last frame, don't timeout */
-	if (msec > 5000)
+	// if in the debugger last frame, don't timeout.
+	if (timedelta > 5000000)
 	{
 		cls.netchan.last_received = Sys_Milliseconds();
 	}
 
+	// Reset power shield / power screen sound counter.
+	num_power_sounds = 0;
+
 	if (!cl_timedemo->value)
 	{
-		// Don't flood while connecting
-		if ((cls.state == ca_connected) && (packetdelta < 100))
+		// Don't throttle too much when connecting / loading.
+		if ((cls.state == ca_connected) && (packetdelta > 100000))
 		{
-			packetframe = false;
-		}
-
-		if (cl_async->value)
-		{
-			// Network frames
-			if (packetdelta < (1000.0f / nfps))
-			{
-				packetframe = false;
-			}
-			else if (cls.nframetime == cls.rframetime)
-			{
-				packetframe = false;
-			}
-
-			// Render frames
-			if (renderdelta < (1000.0f / rfps))
-			{
-				renderframe = false;
-			}
-
-			// Misc. stuff at 10 FPS
-			if (miscdelta < 100.0f)
-			{
-				miscframe = false;
-			}
-		}
-		else
-		{
-			// Cap frames at gl_maxfps
-			if (renderdelta < (1000.0f / rfps))
-			{
-				renderframe = false;
-				packetframe = false;
-				miscframe = false;
-			}
-		}
-
-		// Throttle the game a little bit. 1000 FPS are enough.
-		if (!packetframe && !renderframe && !cls.forcePacket && !userinfo_modified)
-		{
-			double frametime = (1000.0 / cl_maxfps->value - packetdelta) <= (1000.0 / gl_maxfps->value - renderdelta) ?
-							   (1000.0 / cl_maxfps->value - packetdelta) : (1000.0 / gl_maxfps->value - renderdelta);
-
-			if (frametime > 1) // FIXME: why > ??
-			{
-				Sys_Sleep(1);
-			}
-
-			return;
+			packetframe = true;
 		}
 	}
-	else if (msec < 1)
+
+	// Run HTTP downloads more often while connecting.
+#ifdef USE_CURL
+	if (cls.state == ca_connected)
 	{
-		return;
+		CL_RunHTTPDownloads();
 	}
+#endif
 
-	// Update input stuff
+	// Update input stuff.
 	if (packetframe || renderframe)
 	{
 		CL_ReadPackets();
 		CL_UpdateWindowedMouse();
-		Sys_SendKeyEvents();
+		IN_Update();
 		Cbuf_Execute();
 		CL_FixCvarCheats();
 
@@ -870,23 +831,18 @@ CL_Frame(int msec)
 
 	if (packetframe)
 	{
-		packetdelta = 0;
-
 		CL_SendCmd();
 		CL_CheckForResend();
+
+		// Run HTTP downloads during game.
+#ifdef USE_CURL
+		CL_RunHTTPDownloads();
+#endif
 	}
 
 	if (renderframe)
 	{
-		renderdelta = 0;
-
-		if (miscframe)
-		{
-			miscdelta = 0;
-
-			VID_CheckChanges();
-		}
-
+		VID_CheckChanges();
 		CL_PredictMovement();
 
 		if (!cl.refresh_prepped && (cls.state == ca_active))
@@ -910,13 +866,6 @@ CL_Frame(int msec)
 		/* update audio */
 		S_Update(cl.refdef.vieworg, cl.v_forward, cl.v_right, cl.v_up);
 
-#ifdef CDA
-		if (miscframe)
-		{
-			CDAudio_Update();
-		}
-#endif
-
 		/* advance local effects for next frame */
 		CL_RunDLights();
 		CL_RunLightStyles();
@@ -983,12 +932,12 @@ CL_Init(void)
 
 	M_Init();
 
-	cls.disable_screen = true; /* don't draw yet */
-
-#ifdef CDA
-	CDAudio_Init();
+#ifdef USE_CURL
+	CL_InitHTTPDownloads();
 #endif
 
+	cls.disable_screen = true; /* don't draw yet */
+
 	CL_InitLocal();
 
 	Cbuf_Execute();
@@ -1009,18 +958,17 @@ CL_Shutdown(void)
 
 	isdown = true;
 
+#ifdef USE_CURL
+	CL_HTTP_Cleanup(true);
+#endif
+
 	CL_WriteConfiguration();
 
 	Key_WriteConsoleHistory();
 
-#ifdef CDA
-	CDAudio_Shutdown();
-#endif
-#ifdef OGG
 	OGG_Stop();
-#endif
+
 	S_Shutdown();
 	IN_Shutdown();
 	VID_Shutdown();
 }
-
