@@ -50,9 +50,9 @@ cvar_t *scr_graphscale;
 cvar_t *scr_graphshift;
 cvar_t *scr_drawall;
 
-cvar_t *gl_hudscale; /* named for consistency with R1Q2 */
-cvar_t *gl_consolescale;
-cvar_t *gl_menuscale;
+cvar_t *r_hudscale; /* named for consistency with R1Q2 */
+cvar_t *r_consolescale;
+cvar_t *r_menuscale;
 
 typedef struct
 {
@@ -64,7 +64,7 @@ dirty_t scr_dirty, scr_old_dirty[2];
 char crosshair_pic[MAX_QPATH];
 int crosshair_width, crosshair_height;
 
-extern cvar_t *cl_drawfps;
+extern cvar_t *cl_showfps;
 extern cvar_t *crosshair_scale;
 
 void SCR_TimeRefresh_f(void);
@@ -431,9 +431,9 @@ SCR_Init(void)
 	scr_graphscale = Cvar_Get("graphscale", "1", 0);
 	scr_graphshift = Cvar_Get("graphshift", "0", 0);
 	scr_drawall = Cvar_Get("scr_drawall", "0", 0);
-	gl_hudscale = Cvar_Get("gl_hudscale", "-1", CVAR_ARCHIVE);
-	gl_consolescale = Cvar_Get("gl_consolescale", "-1", CVAR_ARCHIVE);
-	gl_menuscale = Cvar_Get("gl_menuscale", "-1", CVAR_ARCHIVE);
+	r_hudscale = Cvar_Get("r_hudscale", "-1", CVAR_ARCHIVE);
+	r_consolescale = Cvar_Get("r_consolescale", "-1", CVAR_ARCHIVE);
+	r_menuscale = Cvar_Get("r_menuscale", "-1", CVAR_ARCHIVE);
 
 	/* register our commands */
 	Cmd_AddCommand("timerefresh", SCR_TimeRefresh_f);
@@ -499,6 +499,13 @@ SCR_DrawLoading(void)
 void
 SCR_RunConsole(void)
 {
+	/* src_conspeed must be a positiv integer,
+	   otherwise things go wrong. Clamp it. */
+	if (scr_conspeed->value < 0.1f)
+	{
+		Cvar_Set("scr_conspeed", "0.1");
+	}
+
 	/* decide on the height of the console */
 	if (cls.key_dest == key_console)
 	{
@@ -567,12 +574,8 @@ SCR_BeginLoadingPlaque(void)
 {
 	S_StopAllSounds();
 	cl.sound_prepped = false; /* don't play ambients */
-#ifdef CDA
-	CDAudio_Stop();
-#endif
-#ifdef OGG
+
 	OGG_Stop();
-#endif
 
 	if (cls.disable_screen)
 	{
@@ -581,6 +584,16 @@ SCR_BeginLoadingPlaque(void)
 
 	if (developer->value)
 	{
+		/* Hack: When we are returning here (not drawing
+		   the loading plaque) we don't reset the palette
+		   later on. We might end up with the cinematic
+		   palette applied to the world. Enforce the world
+		   palette. */
+		if (cl.cinematictime > 0)
+		{
+			R_SetPalette(NULL);
+		}
+
 		return;
 	}
 
@@ -731,93 +744,115 @@ SCR_TileClear(void)
 		return; /* full screen cinematic */
 	}
 
-	/* erase rect will be the union of the past three
-	   frames so tripple buffering works properly */
-	clear = scr_dirty;
 
-	for (i = 0; i < 2; i++)
-	{
-		if (scr_old_dirty[i].x1 < clear.x1)
+	/* This highly complicated pseudo damage tracking is very effective
+	   with the soft render, it gives about 10% speedup. On the GL
+	   renderers the speedup is negligible, but in introduce a bug:
+	   OpenGL requires to redraw the borders after every glClear(),
+	   which the damage tracking doesn't take into account. Fix this
+	   by not using the damage tracking when running somethinge else
+	   than the soft renderer. Just redraw the borders every frame. */
+	if (strcmp(vid_renderer->string, "soft") != 0) {
+		// Top
+		Draw_TileClear(scr_vrect.x, 0, scr_vrect.width, scr_vrect.y, "backtile");
+
+		// Bottom
+		Draw_TileClear(scr_vrect.x, scr_vrect.y + scr_vrect.height, scr_vrect.width, viddef.height, "backtile");
+
+		// Left
+		Draw_TileClear(0, 0, scr_vrect.x, viddef.height, "backtile");
+
+		// Right
+		Draw_TileClear(scr_vrect.x + scr_vrect.width, 0, viddef.width, viddef.height, "backtile");
+	} else {
+		/* erase rect will be the union of the past three
+		   frames so tripple buffering works properly */
+		clear = scr_dirty;
+
+		for (i = 0; i < 2; i++)
 		{
-			clear.x1 = scr_old_dirty[i].x1;
+			if (scr_old_dirty[i].x1 < clear.x1)
+			{
+				clear.x1 = scr_old_dirty[i].x1;
+			}
+
+			if (scr_old_dirty[i].x2 > clear.x2)
+			{
+				clear.x2 = scr_old_dirty[i].x2;
+			}
+
+			if (scr_old_dirty[i].y1 < clear.y1)
+			{
+				clear.y1 = scr_old_dirty[i].y1;
+			}
+
+			if (scr_old_dirty[i].y2 > clear.y2)
+			{
+				clear.y2 = scr_old_dirty[i].y2;
+			}
 		}
 
-		if (scr_old_dirty[i].x2 > clear.x2)
+		scr_old_dirty[1] = scr_old_dirty[0];
+		scr_old_dirty[0] = scr_dirty;
+
+		scr_dirty.x1 = 9999;
+		scr_dirty.x2 = -9999;
+		scr_dirty.y1 = 9999;
+		scr_dirty.y2 = -9999;
+
+		/* don't bother with anything convered by the console */
+		top = (int)(scr_con_current * viddef.height);
+
+		if (top >= clear.y1)
 		{
-			clear.x2 = scr_old_dirty[i].x2;
+			clear.y1 = top;
 		}
 
-		if (scr_old_dirty[i].y1 < clear.y1)
+		if (clear.y2 <= clear.y1)
 		{
-			clear.y1 = scr_old_dirty[i].y1;
+			return; /* nothing disturbed */
 		}
 
-		if (scr_old_dirty[i].y2 > clear.y2)
+		top = scr_vrect.y;
+		bottom = top + scr_vrect.height - 1;
+		left = scr_vrect.x;
+		right = left + scr_vrect.width - 1;
+
+		if (clear.y1 < top)
 		{
-			clear.y2 = scr_old_dirty[i].y2;
+			/* clear above view screen */
+			i = clear.y2 < top - 1 ? clear.y2 : top - 1;
+			Draw_TileClear(clear.x1, clear.y1,
+					clear.x2 - clear.x1 + 1, i - clear.y1 + 1, "backtile");
+			clear.y1 = top;
 		}
-	}
 
-	scr_old_dirty[1] = scr_old_dirty[0];
-	scr_old_dirty[0] = scr_dirty;
+		if (clear.y2 > bottom)
+		{
+			/* clear below view screen */
+			i = clear.y1 > bottom + 1 ? clear.y1 : bottom + 1;
+			Draw_TileClear(clear.x1, i,
+					clear.x2 - clear.x1 + 1, clear.y2 - i + 1, "backtile");
+			clear.y2 = bottom;
+		}
 
-	scr_dirty.x1 = 9999;
-	scr_dirty.x2 = -9999;
-	scr_dirty.y1 = 9999;
-	scr_dirty.y2 = -9999;
+		if (clear.x1 < left)
+		{
+			/* clear left of view screen */
+			i = clear.x2 < left - 1 ? clear.x2 : left - 1;
+			Draw_TileClear(clear.x1, clear.y1,
+					i - clear.x1 + 1, clear.y2 - clear.y1 + 1, "backtile");
+			clear.x1 = left;
+		}
 
-	/* don't bother with anything convered by the console */
-	top = (int)(scr_con_current * viddef.height);
-
-	if (top >= clear.y1)
-	{
-		clear.y1 = top;
-	}
-
-	if (clear.y2 <= clear.y1)
-	{
-		return; /* nothing disturbed */
-	}
-
-	top = scr_vrect.y;
-	bottom = top + scr_vrect.height - 1;
-	left = scr_vrect.x;
-	right = left + scr_vrect.width - 1;
-
-	if (clear.y1 < top)
-	{
-		/* clear above view screen */
-		i = clear.y2 < top - 1 ? clear.y2 : top - 1;
-		Draw_TileClear(clear.x1, clear.y1,
-				clear.x2 - clear.x1 + 1, i - clear.y1 + 1, "backtile");
-		clear.y1 = top;
-	}
-
-	if (clear.y2 > bottom)
-	{
-		/* clear below view screen */
-		i = clear.y1 > bottom + 1 ? clear.y1 : bottom + 1;
-		Draw_TileClear(clear.x1, i,
-				clear.x2 - clear.x1 + 1, clear.y2 - i + 1, "backtile");
-		clear.y2 = bottom;
-	}
-
-	if (clear.x1 < left)
-	{
-		/* clear left of view screen */
-		i = clear.x2 < left - 1 ? clear.x2 : left - 1;
-		Draw_TileClear(clear.x1, clear.y1,
-				i - clear.x1 + 1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x1 = left;
-	}
-
-	if (clear.x2 > right)
-	{
-		/* clear left of view screen */
-		i = clear.x1 > right + 1 ? clear.x1 : right + 1;
-		Draw_TileClear(i, clear.y1,
-				clear.x2 - i + 1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x2 = right;
+		if (clear.x2 > right)
+		{
+			/* clear left of view screen */
+			i = clear.x1 > right + 1 ? clear.x1 : right + 1;
+			Draw_TileClear(i, clear.y1,
+					clear.x2 - i + 1, clear.y2 - clear.y1 + 1, "backtile");
+			clear.x2 = right;
+		}
 	}
 }
 
@@ -1096,7 +1131,7 @@ SCR_ExecuteLayoutString(char *s)
 			token = COM_Parse(&s);
 			index = (int)strtol(token, (char **)NULL, 10);
 
-			if ((index < 0) || (index >= sizeof(cl.frame.playerstate.stats)))
+			if ((index < 0) || (index >= MAX_STATS))
 			{
 				Com_Error(ERR_DROP, "bad stats index %d (0x%x)", index, index);
 			}
@@ -1108,7 +1143,7 @@ SCR_ExecuteLayoutString(char *s)
 				Com_Error(ERR_DROP, "Pic >= MAX_IMAGES");
 			}
 
-			if (cl.configstrings[CS_IMAGES + value])
+			if (cl.configstrings[CS_IMAGES + value][0] != '\0')
 			{
 				SCR_AddDirtyPoint(x, y);
 				SCR_AddDirtyPoint(x + 23*scale, y + 23*scale);
@@ -1207,7 +1242,7 @@ SCR_ExecuteLayoutString(char *s)
 
 			else
 			{
-				DrawAltStringScaled(x, y, block, scale);
+				DrawStringScaled(x, y, block, scale);
 			}
 
 			continue;
@@ -1323,7 +1358,7 @@ SCR_ExecuteLayoutString(char *s)
 			token = COM_Parse(&s);
 			index = (int)strtol(token, (char **)NULL, 10);
 
-			if ((index < 0) || (index >= MAX_CONFIGSTRINGS))
+			if ((index < 0) || (index >= MAX_STATS))
 			{
 				Com_Error(ERR_DROP, "Bad stat_string index");
 			}
@@ -1410,6 +1445,94 @@ SCR_DrawLayout(void)
 	SCR_ExecuteLayoutString(cl.layout);
 }
 
+// ----
+
+void
+SCR_Framecounter(void) {
+	long long newtime;
+	static int frame;
+	static int frametimes[60] = {0};
+	static long long oldtime;
+
+	/* skip statistics without show fps */
+	if (cl_showfps->value < 1)
+		return;
+
+	newtime = Sys_Microseconds();
+	frametimes[frame] = (int)(newtime - oldtime);
+
+	oldtime = newtime;
+	frame++;
+	if (frame > 59) {
+		frame = 0;
+	}
+
+	float scale = SCR_GetConsoleScale();
+
+	if (cl_showfps->value == 1) {
+		// Calculate average of frames.
+		int avg = 0;
+		int num = 0;
+
+		for (int i = 0; i < 60; i++) {
+			if (frametimes[i] != 0) {
+				avg += frametimes[i];
+				num++;
+			}
+		}
+
+
+		char str[10];
+		snprintf(str, sizeof(str), "%3.2ffps", (1000.0 * 1000.0) / (avg / num));
+		DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), 0, str, scale);
+		SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), 0);
+		SCR_AddDirtyPoint(viddef.width, 0);
+	} else if (cl_showfps->value >= 2) {
+		// Calculate average of frames.
+		int avg = 0;
+		int num = 0;
+
+		for (int i = 0; i < 60; i++) {
+			if (frametimes[i] != 0) {
+				avg += frametimes[i];
+				num++;
+			}
+		}
+
+
+		// Find lowest and highest
+		int min = frametimes[0];
+		int max = frametimes[1];
+
+		for (int i = 1; i < 60; i++) {
+			if ((frametimes[i] > 0) &&  (min < frametimes[i])) {
+				min = frametimes[i];
+			}
+
+			if ((frametimes[i] > 0) && (max > frametimes[i])) {
+				max = frametimes[i];
+			}
+		}
+
+		char str[64];
+		snprintf(str, sizeof(str), "Min: %7.2ffps, Max: %7.2ffps, Avg: %7.2ffps",
+		         (1000.0 * 1000.0) / min, (1000.0 * 1000.0) / max, (1000.0 * 1000.0) / (avg / num));
+		DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), 0, str, scale);
+		SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), 0);
+		SCR_AddDirtyPoint(viddef.width, 0);
+
+		if (cl_showfps->value > 2)
+		{
+			snprintf(str, sizeof(str), "Max: %5.2fms, Min: %5.2fms, Avg: %5.2fms",
+			         0.001f*min, 0.001f*max, 0.001f*(avg / num));
+			DrawStringScaled(viddef.width - scale*(strlen(str)*8 + 2), scale*10, str, scale);
+			SCR_AddDirtyPoint(viddef.width - scale*(strlen(str)*8 + 2), scale*10);
+			SCR_AddDirtyPoint(viddef.width, scale+10);
+		}
+	}
+}
+
+// ----
 /*
  * This is called every frame, and can also be called
  * explicitly to flush text to the screen.
@@ -1440,12 +1563,12 @@ SCR_UpdateScreen(void)
 		return; /* not initialized yet */
 	}
 
-	if ( gl_stereo->value )
+	if ( gl1_stereo->value )
 	{
 		numframes = 2;
-		separation[0] = -gl_stereo_separation->value / 2;
-		separation[1] = +gl_stereo_separation->value / 2;
-	}		
+		separation[0] = -gl1_stereo_separation->value / 2;
+		separation[1] = +gl1_stereo_separation->value / 2;
+	}
 	else
 	{
 		separation[0] = 0;
@@ -1462,6 +1585,7 @@ SCR_UpdateScreen(void)
 			/* loading plaque over black screen */
 			int w, h;
 
+			R_EndWorldRenderpass();
 			if(i == 0){
 				R_SetPalette(NULL);
 			}
@@ -1486,6 +1610,7 @@ SCR_UpdateScreen(void)
 					cl.cinematicpalette_active = false;
 				}
 
+				R_EndWorldRenderpass();
 				M_Draw();
 			}
 			else if (cls.key_dest == key_console)
@@ -1496,10 +1621,12 @@ SCR_UpdateScreen(void)
 					cl.cinematicpalette_active = false;
 				}
 
+				R_EndWorldRenderpass();
 				SCR_DrawConsole();
 			}
 			else
 			{
+				R_EndWorldRenderpass();
 				SCR_DrawCinematic();
 			}
 		}
@@ -1535,13 +1662,6 @@ SCR_UpdateScreen(void)
 			SCR_DrawNet();
 			SCR_CheckDrawCenterString();
 
-			if (cl_drawfps->value)
-			{
-				char s[8];
-				sprintf(s, "%3.0ffps", 1 / cls.rframetime);
-				DrawString(viddef.width - 64, 0, s);
-			}
-
 			if (scr_timegraph->value)
 			{
 				SCR_DebugGraph(cls.rframetime * 300, 0);
@@ -1563,6 +1683,7 @@ SCR_UpdateScreen(void)
 		}
 	}
 
+	SCR_Framecounter();
 	R_EndFrame();
 }
 
@@ -1653,17 +1774,17 @@ SCR_GetHUDScale(void)
 	{
 		scale = 1;
 	}
-	else if (gl_hudscale->value < 0)
+	else if (r_hudscale->value < 0)
 	{
 		scale = SCR_GetDefaultScale();
 	}
-	else if (gl_hudscale->value == 0) /* HACK: allow scale 0 to hide the HUD */
+	else if (r_hudscale->value == 0) /* HACK: allow scale 0 to hide the HUD */
 	{
 		scale = 0;
 	}
 	else
 	{
-		scale = SCR_ClampScale(gl_hudscale->value);
+		scale = SCR_ClampScale(r_hudscale->value);
 	}
 
 	return scale;
@@ -1678,13 +1799,13 @@ SCR_GetConsoleScale(void)
 	{
 		scale = 1;
 	}
-	else if (gl_consolescale->value < 0)
+	else if (r_consolescale->value < 0)
 	{
 		scale = SCR_GetDefaultScale();
 	}
 	else
 	{
-		scale = SCR_ClampScale(gl_consolescale->value);
+		scale = SCR_ClampScale(r_consolescale->value);
 	}
 
 	return scale;
@@ -1699,13 +1820,13 @@ SCR_GetMenuScale(void)
 	{
 		scale = 1;
 	}
-	else if (gl_menuscale->value < 0)
+	else if (r_menuscale->value < 0)
 	{
 		scale = SCR_GetDefaultScale();
 	}
 	else
 	{
-		scale = SCR_ClampScale(gl_menuscale->value);
+		scale = SCR_ClampScale(r_menuscale->value);
 	}
 
 	return scale;
