@@ -25,6 +25,7 @@
  */
 
 #include "header/client.h"
+#include "sound/header/local.h"
 
 typedef enum
 {
@@ -671,7 +672,6 @@ CL_ParseTEnt(void)
 			{
 				CL_ParticleEffect(pos, dir, 0, 40);
 			}
-
 			else
 			{
 				CL_ParticleEffect(pos, dir, 0xe0, 6);
@@ -687,12 +687,10 @@ CL_ParseTEnt(void)
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_ric1, 1, ATTN_NORM, 0);
 				}
-
 				else if (cnt == 2)
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_ric2, 1, ATTN_NORM, 0);
 				}
-
 				else if (cnt == 3)
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_ric3, 1, ATTN_NORM, 0);
@@ -716,7 +714,38 @@ CL_ParseTEnt(void)
 				CL_ParticleEffect(pos, dir, 0xb0, 40);
 			}
 
-			S_StartSound(pos, 0, 0, cl_sfx_lashit, 1, ATTN_NORM, 0);
+			if (cl_limitsparksounds->value)
+			{
+				num_power_sounds++;
+
+				/* If too many of these sounds are started in one frame
+				 * (for example if the player shoots with the super
+				 * shotgun into the power screen of a Brain) things get
+				 * too loud and OpenAL is forced to scale the volume of
+				 * several other sounds and the background music down.
+				 * That leads to a noticable and annoying drop in the
+				 * overall volume.
+				 *
+				 * Work around that by limiting the number of sounds
+				 * started.
+				 * 16 was choosen by empirical testing.
+				 *
+				 * This was fixed in openal-soft 0.19.0. We're keeping
+				 * the work around hidden behind a cvar and no longer
+				 * limited to OpenAL because a) some Linux distros may
+				 * still ship older openal-soft versions and b) some
+				 * player may like the changed behavior.
+				 */
+				if (num_power_sounds < 16)
+				{
+					S_StartSound(pos, 0, 0, cl_sfx_lashit, 1, ATTN_NORM, 0);
+				}
+			}
+			else
+			{
+				S_StartSound(pos, 0, 0, cl_sfx_lashit, 1, ATTN_NORM, 0);
+			}
+
 			break;
 
 		case TE_SHOTGUN: /* bullet hitting wall */
@@ -736,7 +765,6 @@ CL_ParseTEnt(void)
 			{
 				color = 0x00;
 			}
-
 			else
 			{
 				color = splash_color[r];
@@ -752,12 +780,10 @@ CL_ParseTEnt(void)
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_spark5, 1, ATTN_STATIC, 0);
 				}
-
 				else if (r == 1)
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_spark6, 1, ATTN_STATIC, 0);
 				}
-
 				else
 				{
 					S_StartSound(pos, 0, 0, cl_sfx_spark7, 1, ATTN_STATIC, 0);
@@ -798,12 +824,10 @@ CL_ParseTEnt(void)
 			{
 				ex->ent.angles[1] = 90;
 			}
-
 			else if (dir[1] < 0)
 			{
 				ex->ent.angles[1] = 270;
 			}
-
 			else
 			{
 				ex->ent.angles[1] = 0;
@@ -850,7 +874,6 @@ CL_ParseTEnt(void)
 			{
 				S_StartSound(pos, 0, 0, cl_sfx_watrexp, 1, ATTN_NORM, 0);
 			}
-
 			else
 			{
 				S_StartSound(pos, 0, 0, cl_sfx_grenexp, 1, ATTN_NORM, 0);
@@ -903,7 +926,6 @@ CL_ParseTEnt(void)
 			{
 				ex->ent.model = cl_mod_explo4;
 			}
-
 			else
 			{
 				ex->ent.model = cl_mod_explo4_big;
@@ -925,7 +947,6 @@ CL_ParseTEnt(void)
 			{
 				S_StartSound(pos, 0, 0, cl_sfx_watrexp, 1, ATTN_NORM, 0);
 			}
-
 			else
 			{
 				S_StartSound(pos, 0, 0, cl_sfx_rockexp, 1, ATTN_NORM, 0);
@@ -1024,7 +1045,6 @@ CL_ParseTEnt(void)
 			{
 				CL_BlasterParticles2(pos, dir, 0xd0);
 			}
-
 			else
 			{
 				CL_BlasterParticles2(pos, dir, 0x6f);
@@ -1038,7 +1058,6 @@ CL_ParseTEnt(void)
 			{
 				ex->ent.angles[1] = (float)atan2(dir[1], dir[0]) / M_PI * 180;
 			}
-
 			else if (dir[1] > 0)
 			{
 				ex->ent.angles[1] = 90;
@@ -1048,7 +1067,6 @@ CL_ParseTEnt(void)
 			{
 				ex->ent.angles[1] = 270;
 			}
-
 			else
 			{
 				ex->ent.angles[1] = 0;
@@ -1061,7 +1079,6 @@ CL_ParseTEnt(void)
 			{
 				ex->ent.skinnum = 1;
 			}
-
 			else /* flechette */
 			{
 				ex->ent.skinnum = 2;
@@ -1074,7 +1091,6 @@ CL_ParseTEnt(void)
 			{
 				ex->lightcolor[1] = 1;
 			}
-
 			else
 			{
 				/* flechette */
@@ -1122,15 +1138,7 @@ CL_ParseTEnt(void)
 
 			ex->frames = 15;
 
-			if (type == TE_ROCKET_EXPLOSION_WATER)
-			{
-				S_StartSound(pos, 0, 0, cl_sfx_watrexp, 1, ATTN_NORM, 0);
-			}
-
-			else
-			{
-				S_StartSound(pos, 0, 0, cl_sfx_rockexp, 1, ATTN_NORM, 0);
-			}
+			S_StartSound(pos, 0, 0, cl_sfx_rockexp, 1, ATTN_NORM, 0);
 
 			break;
 
@@ -1638,7 +1646,7 @@ CL_AddPlayerBeams(void)
 
 			if (cl_mod_heatbeam && (b->model == cl_mod_heatbeam))
 			{
-				ent.flags = RF_FULLBRIGHT;
+				ent.flags = RF_FULLBRIGHT|RF_WEAPONMODEL; // DG: fix rogue heatbeam high FOV rendering
 				ent.angles[0] = -pitch;
 				ent.angles[1] = yaw + 180.0f;
 				ent.angles[2] = (float)((cl.time) % 360);
