@@ -47,16 +47,18 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "ref.h"
-#include "vid.h"
+#include "../../common/header/common.h"
+
+#include "../curl/header/download.h"
+#include "../sound/header/sound.h"
+#include "../sound/header/vorbis.h"
+#include "../vid/header/ref.h"
+#include "../vid/header/vid.h"
+
 #include "screen.h"
 #include "keyboard.h"
 #include "console.h"
 
-#include "../sound/header/cdaudio.h"
-#include "../sound/header/sound.h"
-#include "../sound/header/vorbis.h"
-
 typedef struct
 {
 	qboolean		valid; /* cleared if delta parsing was invalid */
@@ -209,7 +211,7 @@ typedef struct
 	keydest_t	key_dest;
 
 	int			framecount;
-	int			realtime; /* always increasing, no clamping, etc */
+	int			realtime; /* always increasing, no clamping, etc, in MS */
 	float		rframetime; /* seconds since last render frame */
 	float		nframetime; /* network frame time */
 
@@ -239,20 +241,40 @@ typedef struct
 	char		downloadname[MAX_OSPATH];
 	int			downloadnumber;
 	dltype_t	downloadtype;
+	size_t		downloadposition;
 	int			downloadpercent;
 
 	/* demo recording info must be here, so it isn't cleared on level change */
 	qboolean	demorecording;
 	qboolean	demowaiting; /* don't record until a non-delta message is received */
 	FILE		*demofile;
+
+#ifdef USE_CURL
+	/* http downloading */
+	dlqueue_t  downloadQueue; /* queues with files to download. */
+	dlhandle_t HTTPHandles[MAX_HTTP_HANDLES]; /* download handles. */
+	char	   downloadServer[512]; /* URL prefix to dowload from .*/
+	char	   downloadServerRetry[512]; /* retry count. */
+	char	   downloadReferer[32]; /* referer string. */
+#endif
 } client_static_t;
 
 extern client_static_t	cls;
 
+/* Evil hack against too many power screen and power
+   shield impact sounds. For example if the player
+   fires his shotgun onto a Brain. */
+extern int num_power_sounds;
+
+/* Even more evil hack against spurious cinematic
+   aborts caused by an unspeakable evil hack right
+   out of the deeps of hell... Aeh... KeyEvent(). */
+extern int abort_cinematic;
+
 /* cvars */
-extern	cvar_t	*gl_stereo_separation;
-extern	cvar_t	*gl_stereo_convergence;
-extern	cvar_t	*gl_stereo;
+extern	cvar_t	*gl1_stereo_separation;
+extern	cvar_t	*gl1_stereo_convergence;
+extern	cvar_t	*gl1_stereo;
 extern	cvar_t	*cl_gun;
 extern	cvar_t	*cl_add_blend;
 extern	cvar_t	*cl_add_lights;
@@ -271,9 +293,10 @@ extern	cvar_t	*cl_anglespeedkey;
 extern	cvar_t	*cl_shownet;
 extern	cvar_t	*cl_showmiss;
 extern	cvar_t	*cl_showclamp;
-extern	cvar_t	*lookspring;
 extern	cvar_t	*lookstrafe;
-extern	cvar_t	*sensitivity;
+extern	cvar_t	*joy_layout;
+extern	cvar_t	*gyro_mode;
+extern	cvar_t	*gyro_turning_axis;
 extern	cvar_t	*m_pitch;
 extern	cvar_t	*m_yaw;
 extern	cvar_t	*m_forward;
@@ -281,10 +304,18 @@ extern	cvar_t	*m_side;
 extern	cvar_t	*freelook;
 extern	cvar_t	*cl_lightlevel;
 extern	cvar_t	*cl_paused;
+extern	cvar_t	*cl_loadpaused;
+extern  cvar_t  *cl_audiopaused;
+extern  cvar_t  *cl_unpaused_scvis;
 extern	cvar_t	*cl_timedemo;
 extern	cvar_t	*cl_vwep;
 extern	cvar_t  *horplus;
 extern	cvar_t	*cin_force43;
+extern	cvar_t	*vid_fullscreen;
+extern  cvar_t  *vid_renderer;
+extern	cvar_t	*cl_kickangles;
+extern  cvar_t  *cl_r1q2_lightstyle;
+extern  cvar_t  *cl_limitsparksounds;
 
 typedef struct
 {
@@ -305,6 +336,8 @@ extern	entity_state_t	cl_parse_entities[MAX_PARSE_ENTITIES];
 extern	netadr_t	net_from;
 extern	sizebuf_t	net_message;
 
+extern qboolean paused_at_load;
+
 void DrawString (int x, int y, char *s);
 void DrawStringScaled(int x, int y, char *s, float factor);
 void DrawAltString (int x, int y, char *s);	/* toggle high bit */
@@ -426,6 +459,7 @@ void CL_GetChallengePacket (void);
 void CL_PingServers_f (void);
 void CL_Snd_Restart_f (void);
 void CL_RequestNextDownload (void);
+void CL_ResetPrecacheCheck (void);
 
 typedef struct
 {
