@@ -29,7 +29,6 @@
 #include "../header/client.h"
 #include "header/qmenu.h"
 
-static void Action_DoEnter(menuaction_s *a);
 static void Action_Draw(menuaction_s *a);
 static void Menu_DrawStatusBar(const char *string);
 static void MenuList_Draw(menulist_s *l);
@@ -39,58 +38,88 @@ static void Slider_Draw(menuslider_s *s);
 static void SpinControl_Draw(menulist_s *s);
 static void SpinControl_DoSlide(menulist_s *s, int dir);
 
-#define RCOLUMN_OFFSET 16
-#define LCOLUMN_OFFSET -16
-
 extern viddef_t viddef;
 
 #define VID_WIDTH viddef.width
 #define VID_HEIGHT viddef.height
 
-void
-Action_DoEnter(menuaction_s *a)
+float
+ClampCvar(float min, float max, float value)
 {
-	if (a->generic.callback)
+	if (value < min)
 	{
-		a->generic.callback(a);
+		return min;
+	}
+
+	if (value > max)
+	{
+		return max;
+	}
+
+	return value;
+}
+
+/*
+=================
+Bitmap_Draw
+=================
+*/
+void Bitmap_Draw(menubitmap_s * item)
+{
+	float scale = SCR_GetMenuScale();
+	int x = 0;
+	int y = 0;
+
+	x = item->generic.x;
+	y = item->generic.y;
+
+	if (((item->generic.flags & QMF_HIGHLIGHT_IF_FOCUS) &&
+		(Menu_ItemAtCursor(item->generic.parent) == item)))
+	{
+		Draw_PicScaled(x * scale, y * scale, item->focuspic, scale);
+	}
+	else if (item->generic.name)
+	{
+		Draw_PicScaled(x * scale, y * scale, ( char * )item->generic.name, scale);
 	}
 }
 
 void
 Action_Draw(menuaction_s *a)
 {
-    float scale = SCR_GetMenuScale();
+	float scale = SCR_GetMenuScale();
+	int x = 0;
+	int y = 0;
+
+	x = a->generic.parent->x + a->generic.x;
+	y = a->generic.parent->y + a->generic.y;
 
 	if (a->generic.flags & QMF_LEFT_JUSTIFY)
 	{
 		if (a->generic.flags & QMF_GRAYED)
 		{
-			Menu_DrawStringDark(a->generic.x + a->generic.parent->x + (LCOLUMN_OFFSET * scale),
-                    a->generic.y + a->generic.parent->y,
-					a->generic.name);
+			Menu_DrawStringDark(x + (LCOLUMN_OFFSET * scale),
+				y, a->generic.name);
 		}
 
 		else
 		{
-			Menu_DrawString(a->generic.x + a->generic.parent->x + (LCOLUMN_OFFSET * scale),
-                    a->generic.y + a->generic.parent->y,
-					a->generic.name);
+			Menu_DrawString(x + (LCOLUMN_OFFSET * scale),
+				y, a->generic.name);
 		}
 	}
 	else
 	{
 		if (a->generic.flags & QMF_GRAYED)
 		{
-			Menu_DrawStringR2LDark(a->generic.x + a->generic.parent->x + (LCOLUMN_OFFSET * scale),
-                    a->generic.y + a->generic.parent->y,
-					a->generic.name);
+			Menu_DrawStringR2LDark(x + (LCOLUMN_OFFSET * scale),
+				y, a->generic.name);
 		}
 
 		else
 		{
-			Menu_DrawStringR2L(a->generic.x + a->generic.parent->x + (LCOLUMN_OFFSET * scale),
-                    a->generic.y + a->generic.parent->y,
-					a->generic.name);
+			Menu_DrawStringR2L(x + (LCOLUMN_OFFSET * scale),
+				y, a->generic.name);
 		}
 	}
 
@@ -118,41 +147,47 @@ Field_Draw(menufield_s *f)
 	int i, n;
 	char tempbuffer[128] = "";
 	float scale = SCR_GetMenuScale();
+	int x = 0;
+	int y = 0;
+
+	x = f->generic.parent->x + f->generic.x;
+	y = f->generic.parent->y + f->generic.y;
 
 	if (f->generic.name)
 	{
-		Menu_DrawStringR2LDark(f->generic.x + f->generic.parent->x +
-			   	LCOLUMN_OFFSET * scale, f->generic.y + f->generic.parent->y,
-				f->generic.name);
+		Menu_DrawStringR2LDark(x + LCOLUMN_OFFSET * scale,
+			y, f->generic.name);
 	}
 
 	n = f->visible_length + 1;
+
 	if (n > sizeof(tempbuffer))
 	{
 		n = sizeof(tempbuffer);
 	}
+
 	Q_strlcpy(tempbuffer, f->buffer + f->visible_offset, n);
 
-	Draw_CharScaled(f->generic.x + f->generic.parent->x + 16 * scale,
-			(f->generic.y + f->generic.parent->y - 4) * scale, 18, scale);
-	Draw_CharScaled(f->generic.x + f->generic.parent->x + 16 * scale,
-			(f->generic.y + f->generic.parent->y + 4) * scale, 24, scale);
+	Draw_CharScaled(x + (16 * scale),
+			(y - 4) * scale, 18, scale);
+	Draw_CharScaled(x + 16 * scale,
+			(y + 4) * scale, 24, scale);
 
-	Draw_CharScaled((f->generic.x + f->generic.parent->x + 24 * scale) + (f->visible_length * 8 * scale),
-            (f->generic.y + f->generic.parent->y - 4) * scale, 20, scale);
-	Draw_CharScaled((f->generic.x + f->generic.parent->x + 24 * scale) + (f->visible_length * 8 * scale),
-            (f->generic.y + f->generic.parent->y + 4) * scale, 26, scale);
+	Draw_CharScaled(x + (24 * scale) + (f->visible_length * (8 * scale)),
+			(y - 4) * scale, 20, scale);
+	Draw_CharScaled(x + (24 * scale) + (f->visible_length * (8 * scale)),
+			(y + 4) * scale, 26, scale);
 
 	for (i = 0; i < f->visible_length; i++)
 	{
-		Draw_CharScaled((f->generic.x + f->generic.parent->x + 24 * scale) + (i * 8 * scale),
-				(f->generic.y + f->generic.parent->y - 4) * scale, 19, scale);
-		Draw_CharScaled((f->generic.x + f->generic.parent->x + 24 * scale) + (i * 8 * scale),
-				(f->generic.y + f->generic.parent->y + 4) * scale, 25, scale);
+		Draw_CharScaled(x + (24 * scale) + (i * 8 * scale),
+				(y - 4) * scale, 19, scale);
+		Draw_CharScaled(x + (24 * scale) + (i * 8 * scale),
+				(y + 4) * scale, 25, scale);
 	}
 
-	Menu_DrawString(f->generic.x + f->generic.parent->x + 24 * scale,
-			f->generic.y + f->generic.parent->y, tempbuffer);
+	Menu_DrawString(x + (24 * scale),
+		y, tempbuffer);
 
 	if (Menu_ItemAtCursor(f->generic.parent) == f)
 	{
@@ -171,14 +206,14 @@ Field_Draw(menufield_s *f)
 		if (((int)(Sys_Milliseconds() / 250)) & 1)
 		{
 			Draw_CharScaled(
-                f->generic.x + f->generic.parent->x + 24 * scale + (offset * 8 * scale),
-                (f->generic.y + f->generic.parent->y) * scale, 11, scale);
+				x + (24 * scale) + (offset * (8 * scale)),
+				y * scale, 11, scale);
 		}
 		else
 		{
 			Draw_CharScaled(
-                f->generic.x + f->generic.parent->x + 24 * scale + (offset * 8 * scale),
-                (f->generic.y + f->generic.parent->y) * scale, ' ', scale);
+				x + (24 * scale) + (offset * (8 * scale)),
+				y * scale, ' ', scale);
 		}
 	}
 }
@@ -298,19 +333,12 @@ Field_Key(menufield_s *f, int key)
 void
 Menu_AddItem(menuframework_s *menu, void *item)
 {
-	if (menu->nitems == 0)
-	{
-		menu->nslots = 0;
-	}
-
 	if (menu->nitems < MAXMENUITEMS)
 	{
 		menu->items[menu->nitems] = item;
 		((menucommon_s *)menu->items[menu->nitems])->parent = menu;
 		menu->nitems++;
 	}
-
-	menu->nslots = Menu_TallySlots(menu);
 }
 
 /*
@@ -321,14 +349,15 @@ Menu_AddItem(menuframework_s *menu, void *item)
 void
 Menu_AdjustCursor(menuframework_s *m, int dir)
 {
-	menucommon_s *citem;
+	menucommon_s *citem = NULL;
 
 	/* see if it's in a valid spot */
 	if ((m->cursor >= 0) && (m->cursor < m->nitems))
 	{
 		if ((citem = Menu_ItemAtCursor(m)) != 0)
 		{
-			if (citem->type != MTYPE_SEPARATOR)
+			if (citem->type != MTYPE_SEPARATOR &&
+				(citem->flags & QMF_INACTIVE) != QMF_INACTIVE)
 			{
 				return;
 			}
@@ -337,48 +366,31 @@ Menu_AdjustCursor(menuframework_s *m, int dir)
 
 	/* it's not in a valid spot, so crawl in the direction
 	   indicated until we find a valid spot */
-	if (dir == 1)
-	{
-		while (1)
+    int cursor = m->nitems;
+
+    while (cursor-- > 0)
+    {
+		citem = Menu_ItemAtCursor(m);
+
+		if (citem)
 		{
-			citem = Menu_ItemAtCursor(m);
-
-			if (citem)
+			if (citem->type != MTYPE_SEPARATOR &&
+			(citem->flags & QMF_INACTIVE) != QMF_INACTIVE)
 			{
-				if (citem->type != MTYPE_SEPARATOR)
-				{
-					break;
-				}
-			}
-
-			m->cursor += dir;
-
-			if (m->cursor >= m->nitems)
-			{
-				m->cursor = 0;
+				break;
 			}
 		}
-	}
-	else
-	{
-		while (1)
+
+		m->cursor += dir;
+
+		if (m->cursor >= m->nitems)
 		{
-			citem = Menu_ItemAtCursor(m);
+			m->cursor = 0;
+		}
 
-			if (citem)
-			{
-				if (citem->type != MTYPE_SEPARATOR)
-				{
-					break;
-				}
-			}
-
-			m->cursor += dir;
-
-			if (m->cursor < 0)
-			{
-				m->cursor = m->nitems - 1;
-			}
+		if (m->cursor < 0)
+		{
+			m->cursor = m->nitems - 1;
 		}
 	}
 }
@@ -392,7 +404,7 @@ Menu_Center(menuframework_s *menu)
 	height = ((menucommon_s *)menu->items[menu->nitems - 1])->y;
 	height += 10;
 
-    menu->y = (VID_HEIGHT / scale - height) / 2;
+	menu->y = (VID_HEIGHT / scale - height) / 2;
 }
 
 void
@@ -419,6 +431,9 @@ Menu_Draw(menuframework_s *menu)
 			case MTYPE_SPINCONTROL:
 				SpinControl_Draw((menulist_s *)menu->items[i]);
 				break;
+			case MTYPE_BITMAP:
+				Bitmap_Draw(( menubitmap_s * )menu->items[i]);
+				break;
 			case MTYPE_ACTION:
 				Action_Draw((menuaction_s *)menu->items[i]);
 				break;
@@ -438,19 +453,20 @@ Menu_Draw(menuframework_s *menu)
 	{
 		menu->cursordraw(menu);
 	}
-	else if (item && (item->type != MTYPE_FIELD))
+	else if (item && (item->type != MTYPE_FIELD) && item->type != MTYPE_BITMAP)
 	{
 		if (item->flags & QMF_LEFT_JUSTIFY)
 		{
 			Draw_CharScaled(menu->x + (item->x / scale - 24 + item->cursor_offset) * scale,
 					(menu->y + item->y) * scale,
-                    12 + ((int)(Sys_Milliseconds() / 250) & 1), scale);
+					12 + ((int)(Sys_Milliseconds() / 250) & 1), scale);
 		}
 		else
 		{
+			// FIXME:: menu->x + (item->x / scale + 24 + item->cursor_offset) * scale
 			Draw_CharScaled(menu->x + (item->cursor_offset) * scale,
-                    (menu->y + item->y) * scale,
-				   	12 + ((int)(Sys_Milliseconds() / 250) & 1), scale);
+					(menu->y + item->y) * scale,
+					12 + ((int)(Sys_Milliseconds() / 250) & 1), scale);
 		}
 	}
 
@@ -485,7 +501,7 @@ Menu_DrawStatusBar(const char *string)
 	if (string)
 	{
 		int l = (int)strlen(string);
-        float col = (VID_WIDTH / 2) - (l*8 / 2) * scale;
+		int col = (VID_WIDTH / 2) - (l*8 / 2) * scale;
 
 		Draw_Fill(0, VID_HEIGHT - 8 * scale, VID_WIDTH, 8 * scale, 4);
 		Menu_DrawString(col, VID_HEIGHT / scale - 8, string);
@@ -558,22 +574,13 @@ Menu_ItemAtCursor(menuframework_s *m)
 qboolean
 Menu_SelectItem(menuframework_s *s)
 {
-	menucommon_s *item = (menucommon_s *)Menu_ItemAtCursor(s);
+	menucommon_s * item = ( menucommon_s * )Menu_ItemAtCursor(s);
 
-	if (item)
-	{
-		switch (item->type)
-		{
-			case MTYPE_FIELD:
-				return Field_DoEnter((menufield_s *)item);
-			case MTYPE_ACTION:
-				Action_DoEnter((menuaction_s *)item);
-				return true;
-			case MTYPE_LIST:
-				return false;
-			case MTYPE_SPINCONTROL:
-				return false;
-		}
+	if (item->callback) {
+
+		item->callback(item);
+
+		return true;
 	}
 
 	return false;
@@ -604,88 +611,71 @@ Menu_SlideItem(menuframework_s *s, int dir)
 	}
 }
 
-int
-Menu_TallySlots(menuframework_s *menu)
-{
-	int i;
-	int total = 0;
-
-	for (i = 0; i < menu->nitems; i++)
-	{
-		if (((menucommon_s *)menu->items[i])->type == MTYPE_LIST)
-		{
-			int nitems = 0;
-			const char **n = ((menulist_s *)menu->items[i])->itemnames;
-
-			while (*n)
-			{
-				nitems++, n++;
-			}
-
-			total += nitems;
-		}
-		else
-		{
-			total++;
-		}
-	}
-
-	return total;
-}
-
 void
 MenuList_Draw(menulist_s *l)
 {
 	const char **n;
+	float scale = SCR_GetMenuScale();
+	int x = 0;
 	int y = 0;
-    float scale = SCR_GetMenuScale();
 
-	Menu_DrawStringR2LDark(l->generic.x + l->generic.parent->x
-			+ LCOLUMN_OFFSET * scale, l->generic.y + l->generic.parent->y,
-			l->generic.name);
+	x = l->generic.parent->x + l->generic.x;
+	y = l->generic.parent->y + l->generic.y;
+
+	Menu_DrawStringR2LDark(x + (LCOLUMN_OFFSET * scale),
+		y, l->generic.name);
 
 	n = l->itemnames;
 
-	Draw_Fill(l->generic.x - 112 + l->generic.parent->x,
-			l->generic.parent->y + l->generic.y +
-			l->curvalue * 10 + 10, 128, 10, 16);
+	Draw_Fill(x - 112,
+		y + l->curvalue * 10 * scale + 10 * scale, 128, 10, 16);
 
 	while (*n)
 	{
-		Menu_DrawStringR2LDark(l->generic.x + l->generic.parent->x +
-			   	LCOLUMN_OFFSET * scale, l->generic.y + l->generic.parent->y +
-			   	y + 10, *n);
+		Menu_DrawStringR2LDark(x + (LCOLUMN_OFFSET * scale),
+			y, *n);
 
 		n++;
-		y += 10;
+		y += 10 * scale;
 	}
 }
 
 void
 Separator_Draw(menuseparator_s *s)
 {
+	int x = 0;
+	int y = 0;
+
+	x = s->generic.parent->x + s->generic.x;
+	y = s->generic.parent->y + s->generic.y;
+
 	if (s->generic.name)
 	{
-		Menu_DrawStringR2LDark(s->generic.x + s->generic.parent->x,
-				s->generic.y + s->generic.parent->y,
-				s->generic.name);
+		Menu_DrawStringR2LDark(x,
+			y, s->generic.name);
 	}
 }
 
 void
 Slider_DoSlide(menuslider_s *s, int dir)
 {
-	s->curvalue += dir;
+	float value = Cvar_VariableValue(s->cvar);
+	float step = 0.1f;
+	float sign = 1.0f;
 
-	if (s->curvalue > s->maxvalue)
+	if (s->slidestep)
 	{
-		s->curvalue = s->maxvalue;
+		step = s->slidestep;
 	}
-	else if (s->curvalue < s->minvalue)
+	if (s->abs && value < 0)	// absolute value treatment
 	{
-		s->curvalue = s->minvalue;
+		value = -value;
+		sign = -1.0f;
 	}
 
+	value += dir * step;
+	Cvar_SetValue(s->cvar, ClampCvar(s->minvalue, s->maxvalue, value) * sign);
+
 	if (s->generic.callback)
 	{
 		s->generic.callback(s);
@@ -698,40 +688,48 @@ void
 Slider_Draw(menuslider_s *s)
 {
 	int i;
+	char buffer[5];
+	const char * format;
 	float scale = SCR_GetMenuScale();
+	int x = s->generic.parent->x + s->generic.x;
+	int y = s->generic.parent->y + s->generic.y;
 
-	Menu_DrawStringR2LDark(s->generic.x + s->generic.parent->x +
-		   	LCOLUMN_OFFSET * scale, s->generic.y + s->generic.parent->y,
-			s->generic.name);
-
-	s->range = (s->curvalue - s->minvalue) /
-		(float)(s->maxvalue - s->minvalue);
-
-	if (s->range < 0)
+	float value = Cvar_VariableValue(s->cvar);
+	if (s->abs && value < 0)	// absolute value
 	{
-		s->range = 0;
+		value = -value;
 	}
+	float range = (ClampCvar(s->minvalue, s->maxvalue, value) - s->minvalue) /
+			(s->maxvalue - s->minvalue);
 
-	if (s->range > 1)
-	{
-		s->range = 1;
-	}
+	Menu_DrawStringR2LDark(x + (LCOLUMN_OFFSET * scale),
+		y, s->generic.name);
 
-	Draw_CharScaled(s->generic.x + (s->generic.parent->x + RCOLUMN_OFFSET * scale),
-			(s->generic.y + s->generic.parent->y) * scale, 128, scale);
+	Draw_CharScaled(x + (RCOLUMN_OFFSET * scale),
+		y * scale, 128, scale);
 
 	for (i = 0; i < SLIDER_RANGE * scale; i++)
 	{
-		Draw_CharScaled((RCOLUMN_OFFSET * scale + s->generic.x + i * 8 + s->generic.parent->x + 8),
-			   	(s->generic.y + s->generic.parent->y) * scale, 129, scale);
+		Draw_CharScaled(x + (RCOLUMN_OFFSET * scale) + (i * 8) + 8,
+			y * scale, 129, scale);
 	}
 
-	Draw_CharScaled((RCOLUMN_OFFSET * scale + s->generic.x + i * 8 +
-			s->generic.parent->x + 8), (s->generic.y +
-		   	s->generic.parent->y) * scale, 130, scale);
-	Draw_CharScaled(((int)(8 + RCOLUMN_OFFSET * scale + s->generic.parent->x +
-			s->generic.x + (SLIDER_RANGE * scale - 1) * 8 * s->range)),
-			(s->generic.y + s->generic.parent->y) * scale, 131, scale);
+	Draw_CharScaled(x + (RCOLUMN_OFFSET * scale) + (i * 8) + 8,
+		y * scale, 130, scale);
+	Draw_CharScaled(x + ((int)((RCOLUMN_OFFSET * scale) + (SLIDER_RANGE * scale - 1) * 8 * range)) + 8,
+		y * scale, 131, scale);
+
+	if (!s->printformat)
+	{
+		format = "%.1f";
+	}
+	else
+	{
+		format = s->printformat;
+	}
+	snprintf(buffer, 5, format, value);
+	Menu_DrawString(x + (RCOLUMN_OFFSET * scale) + ((SLIDER_RANGE + 2) * scale * 8),
+		y, buffer);
 }
 
 void
@@ -742,10 +740,12 @@ SpinControl_DoSlide(menulist_s *s, int dir)
 	if (s->curvalue < 0)
 	{
 		s->curvalue = 0;
+		return;
 	}
 	else if (s->itemnames[s->curvalue] == 0)
 	{
 		s->curvalue--;
+		return;
 	}
 
 	if (s->generic.callback)
@@ -758,33 +758,33 @@ void
 SpinControl_Draw(menulist_s *s)
 {
 	char buffer[100];
-    float scale = SCR_GetMenuScale();
+	float scale = SCR_GetMenuScale();
+	int x = 0;
+	int y = 0;
+
+	x = s->generic.parent->x + s->generic.x;
+	y = s->generic.parent->y + s->generic.y;
 
 	if (s->generic.name)
 	{
-		Menu_DrawStringR2LDark(s->generic.x + s->generic.parent->x +
-			   	LCOLUMN_OFFSET * scale, s->generic.y + s->generic.parent->y,
-				s->generic.name);
+		Menu_DrawStringR2LDark(x + (LCOLUMN_OFFSET * scale),
+			y, s->generic.name);
 	}
 
 	if (!strchr(s->itemnames[s->curvalue], '\n'))
 	{
-		Menu_DrawString(RCOLUMN_OFFSET * scale + s->generic.x +
-				s->generic.parent->x, s->generic.y +
-				s->generic.parent->y,
-				s->itemnames[s->curvalue]);
+		Menu_DrawString(x + (RCOLUMN_OFFSET * scale),
+			y, s->itemnames[s->curvalue]);
 	}
 	else
 	{
 		strcpy(buffer, s->itemnames[s->curvalue]);
 		*strchr(buffer, '\n') = 0;
-		Menu_DrawString(RCOLUMN_OFFSET * scale + s->generic.x +
-			   	s->generic.parent->x, s->generic.y +
-				s->generic.parent->y, buffer);
+		Menu_DrawString(x + (RCOLUMN_OFFSET * scale),
+			y, buffer);
 		strcpy(buffer, strchr(s->itemnames[s->curvalue], '\n') + 1);
-		Menu_DrawString(RCOLUMN_OFFSET * scale + s->generic.x +
-				s->generic.parent->x, s->generic.y +
-				s->generic.parent->y + 10, buffer);
+		Menu_DrawString(x + (RCOLUMN_OFFSET * scale),
+			y + 10, buffer);
 	}
 }
 
