@@ -32,17 +32,13 @@
 #include "shared.h"
 #include "crc.h"
 
-#define YQ2VERSION "7.01"
+#define YQ2VERSION "8.30"
 #define BASEDIRNAME "baseq2"
 
 #ifndef YQ2OSTYPE
 #error YQ2OSTYPE should be defined by the build system
 #endif
 
-#ifndef YQ2ARCH
-#error YQ2ARCH should be defined by the build system
-#endif
-
 #ifndef BUILD_DATE
 #define BUILD_DATE __DATE__
 #endif
@@ -50,18 +46,42 @@
 #ifdef _WIN32
  #define CFGDIR "YamagiQ2"
 #else
- #define CFGDIR ".yq2"
+ #ifndef __HAIKU__
+   #define CFGDIR ".yq2"
+ #else
+   #define CFGDIR "yq2"
+ #endif
 #endif
 
-#ifdef _WIN32
- #define LIBGL "opengl32.dll"
-#elif defined __OpenBSD__
- #define LIBGL "libGL.so"
-#elif defined __APPLE__
- #define LIBGL "/System/Library/Frameworks/OpenGL.framework/OpenGL"
-#else
- #define LIBGL "libGL.so.1"
-#endif
+#ifndef YQ2ARCH
+  #ifdef _MSC_VER
+    // Setting YQ2ARCH for VisualC++ from CMake doesn't work when using VS integrated CMake
+    // so set it in code instead
+    #ifdef YQ2ARCH
+      #undef YQ2ARCH
+    #endif
+    #ifdef _M_X64
+      // this matches AMD64 and ARM64EC (but not regular ARM64), but they're supposed to be binary-compatible somehow, so whatever
+      #define YQ2ARCH "x86_64"
+    #elif defined(_M_ARM64)
+      #define YQ2ARCH "arm64"
+    #elif defined(_M_ARM)
+      #define YQ2ARCH "arm"
+    #elif defined(_M_IX86)
+      #define YQ2ARCH "x86"
+    #else
+      // if you're not targeting one of the aforementioned architectures,
+      // check https://learn.microsoft.com/en-us/cpp/preprocessor/predefined-macros
+      // to find out how to detect yours and add it here - and please send a patch :)
+      #error "Unknown CPU architecture!"
+      // (for a quick and dirty solution, comment out the previous line, but keep in mind
+      //  that savegames may not be compatible with other builds of Yamagi Quake II)
+      #define YQ2ARCH "UNKNOWN"
+    #endif // _M_X64 etc
+  #else // other compilers than MSVC
+    #error YQ2ARCH should be defined by the build system
+  #endif // _MSC_VER
+#endif // YQ2ARCH
 
 /* ================================================================== */
 
@@ -147,7 +167,7 @@ void COM_AddParm(char *parm);
 void COM_Init(void);
 void COM_InitArgv(int argc, char **argv);
 
-char *CopyString(char *in);
+char *CopyString(const char *in);
 
 /* ================================================================== */
 
@@ -365,6 +385,7 @@ void Cbuf_InsertFromDefer(void);
 typedef void (*xcommand_t)(void);
 
 void Cmd_Init(void);
+void Cmd_Shutdown(void);
 
 void Cmd_AddCommand(char *cmd_name, xcommand_t function);
 
@@ -381,6 +402,8 @@ qboolean Cmd_Exists(char *cmd_name);
 
 char *Cmd_CompleteCommand(char *partial);
 
+char *Cmd_CompleteMapCommand(char *partial);
+
 /* attempts to match a partial command for automatic command line completion */
 /* returns NULL if nothing fits */
 
@@ -425,27 +448,27 @@ void Cmd_ForwardToServer(void);
 
 extern cvar_t *cvar_vars;
 
-cvar_t *Cvar_Get(char *var_name, char *value, int flags);
+cvar_t *Cvar_Get(const char *var_name, char *value, int flags);
 
 /* creates the variable if it doesn't exist, or returns the existing one */
 /* if it exists, the value will not be changed, but flags will be ORed in */
 /* that allows variables to be unarchived without needing bitflags */
 
-cvar_t *Cvar_Set(char *var_name, char *value);
+cvar_t *Cvar_Set(const char *var_name, char *value);
 
 /* will create the variable if it doesn't exist */
 
-cvar_t *Cvar_ForceSet(char *var_name, char *value);
+cvar_t *Cvar_ForceSet(const char *var_name, char *value);
 
 /* will set the variable even if NOSET or LATCH */
 
-cvar_t *Cvar_FullSet(char *var_name, char *value, int flags);
+cvar_t *Cvar_FullSet(const char *var_name, char *value, int flags);
 
-void Cvar_SetValue(char *var_name, float value);
+void Cvar_SetValue(const char *var_name, float value);
 
 /* expands value to a string and calls Cvar_Set */
 
-float Cvar_VariableValue(char *var_name);
+float Cvar_VariableValue(const char *var_name);
 
 /* returns 0 if not defined or non numeric */
 
@@ -468,13 +491,15 @@ qboolean Cvar_Command(void);
 /* command.  Returns true if the command was a variable reference that */
 /* was handled. (print or change) */
 
-void Cvar_WriteVariables(char *path);
+void Cvar_WriteVariables(const char *path);
 
 /* appends lines containing "set variable value" for all variables */
 /* with the archive flag set to true. */
 
 void Cvar_Init(void);
 
+void Cvar_Fini(void);
+
 char *Cvar_Userinfo(void);
 
 /* returns an info string containing all the CVAR_USERINFO cvars */
@@ -529,7 +554,7 @@ qboolean NET_CompareAdr(netadr_t a, netadr_t b);
 qboolean NET_CompareBaseAdr(netadr_t a, netadr_t b);
 qboolean NET_IsLocalAddress(netadr_t adr);
 char *NET_AdrToString(netadr_t a);
-qboolean NET_StringToAdr(char *s, netadr_t *a);
+qboolean NET_StringToAdr(const char *s, netadr_t *a);
 void NET_Sleep(int msec);
 
 /*=================================================================== */
@@ -643,8 +668,6 @@ void Pmove(pmove_t *pmove);
 
 #define SFF_INPACK 0x20
 
-extern int file_from_pak;
-
 typedef int fileHandle_t;
 
 typedef enum
@@ -673,6 +696,11 @@ int FS_FOpenFile(const char *name, fileHandle_t *f, qboolean gamedir_only);
 void FS_FCloseFile(fileHandle_t f);
 int FS_Read(void *buffer, int size, fileHandle_t f);
 int FS_FRead(void *buffer, int size, int count, fileHandle_t f);
+
+// returns the filename used to open f, but (if opened from pack) in correct case
+// returns NULL if f is no valid handle
+const char* FS_GetFilenameForHandle(fileHandle_t f);
+
 char **FS_ListFiles(char *findname, int *numfiles,
 		unsigned musthave, unsigned canthave);
 char **FS_ListFiles2(char *findname, int *numfiles,
@@ -680,10 +708,15 @@ char **FS_ListFiles2(char *findname, int *numfiles,
 void FS_FreeList(char **list, int nfiles);
 
 void FS_InitFilesystem(void);
-void FS_SetGamedir(char *dir);
+void FS_ShutdownFilesystem(void);
+void FS_BuildGameSpecificSearchPath(char *dir);
 char *FS_Gamedir(void);
 char *FS_NextPath(char *prevpath);
 int FS_LoadFile(char *path, void **buffer);
+qboolean FS_FileInGamedir(const char *file);
+qboolean FS_AddPAKFromGamedir(const char *pak);
+const char* FS_GetNextRawPath(const char* lastRawPath);
+char **FS_ListMods(int *nummods);
 
 /* a null buffer will just return the file length without loading */
 /* a -1 length is not present */
@@ -706,14 +739,31 @@ void FS_CreatePath(char *path);
 #define PRINT_ALL 0
 #define PRINT_DEVELOPER 1   /* only print when "developer 1" */
 
-void Com_BeginRedirect(int target, char *buffer, int buffersize, void (*flush));
+void Com_BeginRedirect(int target, char *buffer, int buffersize, void (*flush)(int, char *));
 void Com_EndRedirect(void);
-void Com_Printf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
-void Com_DPrintf(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
+void Com_Printf(const char *fmt, ...) PRINTF_ATTR(1, 2);
+void Com_DPrintf(const char *fmt, ...) PRINTF_ATTR(1, 2);
 void Com_VPrintf(int print_level, const char *fmt, va_list argptr); /* print_level is PRINT_ALL or PRINT_DEVELOPER */
-void Com_MDPrintf(char *fmt, ...);
-void Com_Error(int code, char *fmt, ...);
-void Com_Quit(void);
+void Com_MDPrintf(const char *fmt, ...) PRINTF_ATTR(1, 2);
+YQ2_ATTR_NORETURN_FUNCPTR void Com_Error(int code, const char *fmt, ...) PRINTF_ATTR(2, 3);
+YQ2_ATTR_NORETURN void Com_Quit(void);
+
+/* Ugly work around for unsupported
+ * format specifiers unter mingw. */
+#ifdef WIN32
+#define YQ2_COM_PRId64 "%I64d"
+#define YQ2_COM_PRIdS "%Id"
+#else
+#define YQ2_COM_PRId64 "%ld"
+#define YQ2_COM_PRIdS "%zd"
+#endif
+
+// terminate yq2 (with Com_Error()) if VAR is NULL (after malloc() or similar)
+// and print message about it
+#define YQ2_COM_CHECK_OOM(VAR, ALLOC_FN_NAME, ALLOC_SIZE) \
+	if(VAR == NULL) { \
+		Com_Error(ERR_FATAL, "%s for " YQ2_COM_PRIdS " bytes failed in %s() (%s == NULL)! Out of Memory?!\n", \
+		                     ALLOC_FN_NAME, (size_t)ALLOC_SIZE, __func__, #VAR); }
 
 int Com_ServerState(void);              /* this should have just been a cvar... */
 void Com_SetServerState(int state);
@@ -726,7 +776,23 @@ extern cvar_t *modder;
 extern cvar_t *dedicated;
 extern cvar_t *host_speeds;
 extern cvar_t *log_stats;
-extern cvar_t *portable;
+
+/* External entity files. */
+extern cvar_t *sv_entfile;
+
+/* Hack for portable client */
+extern qboolean is_portable;
+
+/* Hack for external datadir */
+extern char datadir[MAX_OSPATH];
+
+/* Hack for external datadir */
+extern char cfgdir[MAX_OSPATH];
+
+/* Hack for working 'game' cmd */
+extern char userGivenGame[MAX_QPATH];
+extern char **mapnames;
+extern int nummaps;
 
 extern FILE *log_stats_file;
 
@@ -742,7 +808,8 @@ void *Z_TagMalloc(int size, int tag);
 void Z_FreeTags(int tag);
 
 void Qcommon_Init(int argc, char **argv);
-void Qcommon_Frame(int msec);
+void Qcommon_ExecConfigs(qboolean addEarlyCmds);
+const char* Qcommon_GetInitialGame(void);
 void Qcommon_Shutdown(void);
 
 #define NUMVERTEXNORMALS 162
@@ -751,37 +818,54 @@ extern vec3_t bytedirs[NUMVERTEXNORMALS];
 /* this is in the client code, but can be used for debugging from server */
 void SCR_DebugGraph(float value, int color);
 
-/* NON-PORTABLE OSTYPE SERVICES */
-
-void Sys_Init(void);
-void Sys_UnloadGame(void);
-void *Sys_GetGameAPI(void *parms);
-
-char *Sys_ConsoleInput(void);
-void Sys_ConsoleOutput(char *string);
-void Sys_SendKeyEvents(void);
-void Sys_Error(char *error, ...);
-void Sys_Quit(void);
-char *Sys_GetHomeDir(void);
-const char *Sys_GetBinaryDir(void);
-void Sys_Sleep(int msec);
-
-void Sys_FreeLibrary(void *handle);
-void *Sys_LoadLibrary(const char *path, const char *sym, void **handle);
-void *Sys_GetProcAddress(void *handle, const char *sym);
-void Sys_RedirectStdout(void);
-
 /* CLIENT / SERVER SYSTEMS */
 
 void CL_Init(void);
 void CL_Drop(void);
 void CL_Shutdown(void);
-void CL_Frame(int msec);
+void CL_Frame(int packetdelta, int renderdelta, int timedelta, qboolean packetframe, qboolean renderframe);
 void Con_Print(char *text);
 void SCR_BeginLoadingPlaque(void);
 
 void SV_Init(void);
 void SV_Shutdown(char *finalmsg, qboolean reconnect);
-void SV_Frame(int msec);
+void SV_Frame(int usec);
+
+/* ======================================================================= */
+
+// Platform specific functions.
+
+// system.c
+char *Sys_ConsoleInput(void);
+void Sys_ConsoleOutput(char *string);
+YQ2_ATTR_NORETURN void Sys_Error(const char *error, ...);
+YQ2_ATTR_NORETURN void Sys_Quit(void);
+void Sys_Init(void);
+char *Sys_GetHomeDir(void);
+void Sys_Remove(const char *path);
+int Sys_Rename(const char *from, const char *to);
+void Sys_RemoveDir(const char *path);
+long long Sys_Microseconds(void);
+void Sys_Nanosleep(int);
+void *Sys_GetProcAddress(void *handle, const char *sym);
+void Sys_FreeLibrary(void *handle);
+void *Sys_LoadLibrary(const char *path, const char *sym, void **handle);
+void *Sys_GetGameAPI(void *parms);
+void Sys_UnloadGame(void);
+void Sys_GetWorkDir(char *buffer, size_t len);
+qboolean Sys_SetWorkDir(char *path);
+qboolean Sys_Realpath(const char *in, char *out, size_t size);
+
+// Windows only (system.c)
+#ifdef _WIN32
+void Sys_RedirectStdout(void);
+void Sys_SetHighDPIMode(void);
+#endif
+
+// misc.c
+const char *Sys_GetBinaryDir(void);
+void Sys_SetupFPU(void);
+
+/* ======================================================================= */
 
 #endif
