@@ -75,7 +75,7 @@ RotatePointAroundVector(vec3_t dst, const vec3_t dir,
 	im[2][1] = m[1][2];
 
 	memset(zrot, 0, sizeof(zrot));
-	zrot[0][0] = zrot[1][1] = zrot[2][2] = 1.0F;
+	zrot[2][2] = 1.0F;
 
 	zrot[0][0] = (float)cos(DEG2RAD(degrees));
 	zrot[0][1] = (float)sin(DEG2RAD(degrees));
@@ -521,20 +521,9 @@ VectorNormalize(vec3_t v)
 vec_t
 VectorNormalize2(vec3_t v, vec3_t out)
 {
-	float length, ilength;
+	VectorCopy(v, out);
 
-	length = v[0] * v[0] + v[1] * v[1] + v[2] * v[2];
-	length = (float)sqrt(length);
-
-	if (length)
-	{
-		ilength = 1 / length;
-		out[0] = v[0] * ilength;
-		out[1] = v[1] * ilength;
-		out[2] = v[2] * ilength;
-	}
-
-	return length;
+	return VectorNormalize(out);
 }
 
 void
@@ -867,9 +856,12 @@ void
 Swap_Init(void)
 {
 	byte swaptest[2] = {1, 0};
+	short swapTestShort;
+	YQ2_STATIC_ASSERT(sizeof(short) == 2, "invalid short size");
+	memcpy(&swapTestShort, swaptest, 2);
 
 	/* set the byte swapping variables in a portable manner */
-	if (*(short *)swaptest == 1)
+	if (swapTestShort == 1)
 	{
 		bigendien = false;
 		_BigShort = ShortSwap;
@@ -892,7 +884,7 @@ Swap_Init(void)
 		Com_Printf("Byte ordering: big endian\n\n");
 	}
 
-	if (LittleShort(*(short *)swaptest) != 1)
+	if (LittleShort(swapTestShort) != 1)
 		assert("Error in the endian conversion!");
 }
 
@@ -901,7 +893,7 @@ Swap_Init(void)
  * need to have varargs versions of all text functions.
  */
 char *
-va(char *format, ...)
+va(const char *format, ...)
 {
 	va_list argptr;
 	static char string[1024];
@@ -1007,7 +999,7 @@ done:
 	return com_token;
 }
 
-int paged_total;
+static int paged_total = 0;
 
 void
 Com_PageInMemory(byte *buffer, int size)
@@ -1031,7 +1023,11 @@ Com_PageInMemory(byte *buffer, int size)
 int
 Q_stricmp(const char *s1, const char *s2)
 {
+#ifdef _MSC_VER
+	return stricmp(s1, s2);
+#else
 	return strcasecmp(s1, s2);
+#endif
 }
 
 int
@@ -1101,7 +1097,7 @@ Q_strlwr ( char *s )
 
 	while ( *s )
 	{
-		*s = tolower( *s );
+		*s = tolower( (unsigned char)*s );
 		s++;
 	}
 
@@ -1144,6 +1140,60 @@ Q_strlcat(char *dst, const char *src, int size)
 	return (d - dst) + Q_strlcpy(d, src, size);
 }
 
+/*
+ * An unicode compatible fopen() Wrapper for Windows.
+ */
+#ifdef _WIN32
+#include <windows.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <errno.h>
+FILE *Q_fopen(const char *file, const char *mode)
+{
+	WCHAR wfile[MAX_OSPATH];
+	WCHAR wmode[16];
+
+	int len = MultiByteToWideChar(CP_UTF8, 0, file, -1, wfile, MAX_OSPATH);
+
+	if (len > 0)
+	{
+		if (MultiByteToWideChar(CP_UTF8, 0, mode, -1, wmode, 16) > 0)
+		{
+			// make sure it's a regular file and not a directory or sth, see #394
+			struct _stat buf;
+			int statret = _wstat(wfile, &buf);
+			if((statret == 0 && (buf.st_mode & _S_IFREG) != 0) || (statret == -1 && errno == ENOENT))
+			{
+				return _wfopen(wfile, wmode);
+			}
+		}
+	}
+
+	return NULL;
+}
+#else
+#include <sys/stat.h>
+#include <errno.h>
+FILE *Q_fopen(const char *file, const char *mode)
+{
+	// make sure it's a regular file and not a directory or sth, see #394
+	struct stat statbuf;
+	int statret = stat(file, &statbuf);
+	// (it's ok if it doesn't exist though, maybe we wanna write/create)
+	if((statret == -1 && errno != ENOENT) || (statret == 0 && (statbuf.st_mode & S_IFREG) == 0))
+	{
+		return NULL;
+	}
+	return fopen(file, mode);
+}
+#endif
+
+int
+Q_sort_strcomp(const void *s1, const void *s2)
+{
+	return strcmp(*(char **)s1, *(char **)s2);
+}
+
 /*
  * =====================================================================
  *
@@ -1194,11 +1244,6 @@ Info_ValueForKey(char *s, char *key)
 
 		while (*s != '\\' && *s)
 		{
-			if (!*s)
-			{
-				return "";
-			}
-
 			*o++ = *s++;
 		}
 
@@ -1259,11 +1304,6 @@ Info_RemoveKey(char *s, char *key)
 
 		while (*s != '\\' && *s)
 		{
-			if (!*s)
-			{
-				return;
-			}
-
 			*o++ = *s++;
 		}
 
@@ -1309,7 +1349,12 @@ Info_SetValueForKey(char *s, char *key, char *value)
 	int c;
 	int maxsize = MAX_INFO_STRING;
 
-	if (strstr(key, "\\") || strstr(value, "\\"))
+	if (!key)
+	{
+		return;
+	}
+
+	if (strstr(key, "\\") || (value && strstr(value, "\\")))
 	{
 		Com_Printf("Can't use keys or values with a \\\n");
 		return;
@@ -1317,17 +1362,17 @@ Info_SetValueForKey(char *s, char *key, char *value)
 
 	if (strstr(key, ";"))
 	{
-		Com_Printf("Can't use keys or values with a semicolon\n");
+		Com_Printf("Can't use keys with a semicolon\n");
 		return;
 	}
 
-	if (strstr(key, "\"") || strstr(value, "\""))
+	if (strstr(key, "\"") || (value && strstr(value, "\"")))
 	{
 		Com_Printf("Can't use keys or values with a \"\n");
 		return;
 	}
 
-	if ((strlen(key) > MAX_INFO_KEY - 1) || (strlen(value) > MAX_INFO_KEY - 1))
+	if ((strlen(key) > MAX_INFO_KEY - 1) || (value && (strlen(value) > MAX_INFO_KEY - 1)))
 	{
 		Com_Printf("Keys and values must be < 64 characters.\n");
 		return;
@@ -1342,7 +1387,7 @@ Info_SetValueForKey(char *s, char *key, char *value)
 
 	Com_sprintf(newi, sizeof(newi), "\\%s\\%s", key, value);
 
-	if (strlen(newi) + strlen(s) > maxsize)
+	if (strlen(newi) + strlen(s) >= maxsize)
 	{
 		Com_Printf("Info string length exceeded\n");
 		return;
