@@ -26,8 +26,8 @@
 
 #include "header/local.h"
 
+int debristhisframe;
 int gibsthisframe;
-int lastgibframe;
 
 void
 Use_Areaportal(edict_t *ent, edict_t *other /* unused */, edict_t *activator /* unused */)
@@ -138,7 +138,7 @@ gib_think(edict_t *self)
 void
 gib_touch(edict_t *self, edict_t *other /* unused */, cplane_t *plane, csurface_t *surf /* unused */)
 {
-	if (!self || !plane)
+	if (!self)
 	{
 		return;
 	}
@@ -196,20 +196,19 @@ ThrowGib(edict_t *self, char *gibname, int damage, int type)
 		return;
 	}
 
-	if (level.framenum > lastgibframe)
-	{
-		gibsthisframe = 0;
-		lastgibframe = level.framenum;
-	}
-
-	gibsthisframe++;
-
-	if (gibsthisframe > 20)
+	if (gibsthisframe >= MAX_GIBS)
 	{
 		return;
 	}
 
-	gib = G_Spawn();
+	gib = G_SpawnOptional();
+
+	if (!gib)
+	{
+		return;
+	}
+
+	gibsthisframe++;
 
 	VectorScale(self->size, 0.5, size);
 	VectorAdd(self->absmin, size, origin);
@@ -218,11 +217,13 @@ ThrowGib(edict_t *self, char *gibname, int damage, int type)
 	gib->s.origin[2] = origin[2] + crandom() * size[2];
 
 	gi.setmodel(gib, gibname);
-	gib->solid = SOLID_NOT;
+	gib->solid = SOLID_BBOX;
+	gib->svflags = SVF_DEADMONSTER;
 	gib->s.effects |= EF_GIB;
 	gib->flags |= FL_NO_KNOCKBACK;
 	gib->takedamage = DAMAGE_YES;
 	gib->die = gib_die;
+	gib->health = 250;
 
 	if (type == GIB_ORGANIC)
 	{
@@ -267,7 +268,7 @@ ThrowHead(edict_t *self, char *gibname, int damage, int type)
 
 	self->s.modelindex2 = 0;
 	gi.setmodel(self, gibname);
-	self->solid = SOLID_NOT;
+	self->solid = SOLID_BBOX;
 	self->s.effects |= EF_GIB;
 	self->s.effects &= ~EF_FLIES;
 	self->s.sound = 0;
@@ -277,6 +278,11 @@ ThrowHead(edict_t *self, char *gibname, int damage, int type)
 	self->targetname = NULL;
 	self->die = gib_die;
 
+	// The entity still has the monsters clipmaks.
+	// Reset it to MASK_SHOT to be on the save side.
+	// (MASK_SHOT is used by xatrix)
+	self->clipmask = MASK_SHOT;
+
 	if (type == GIB_ORGANIC)
 	{
 		self->movetype = MOVETYPE_TOSS;
@@ -330,11 +336,16 @@ ThrowClientHead(edict_t *self, int damage)
 	VectorSet(self->maxs, 16, 16, 16);
 
 	self->takedamage = DAMAGE_NO;
-	self->solid = SOLID_NOT;
+	self->solid = SOLID_BBOX;
 	self->s.effects = EF_GIB;
 	self->s.sound = 0;
 	self->flags |= FL_NO_KNOCKBACK;
 
+	// The entity still has the monsters clipmaks.
+	// Reset it to MASK_SHOT to be on the save side.
+	// (MASK_SHOT is used by xatrix)
+	self->clipmask = MASK_SHOT;
+
 	self->movetype = MOVETYPE_BOUNCE;
 	VelocityForDamage(damage, vd);
 	VectorAdd(self->velocity, vd, self->velocity);
@@ -378,20 +389,20 @@ ThrowDebris(edict_t *self, char *modelname, float speed, vec3_t origin)
 		return;
 	}
 
-	if (level.framenum > lastgibframe)
-	{
-		gibsthisframe = 0;
-		lastgibframe = level.framenum;
-	}
-
-	gibsthisframe++;
-
-	if (gibsthisframe > 20)
+	if (debristhisframe >= MAX_DEBRIS)
 	{
 		return;
 	}
 
-	chunk = G_Spawn();
+	chunk = G_SpawnOptional();
+
+	if (!chunk)
+	{
+		return;
+	}
+
+	debristhisframe++;
+
 	VectorCopy(origin, chunk->s.origin);
 	gi.setmodel(chunk, modelname);
 	v[0] = 100 * crandom();
@@ -410,6 +421,7 @@ ThrowDebris(edict_t *self, char *modelname, float speed, vec3_t origin)
 	chunk->classname = "debris";
 	chunk->takedamage = DAMAGE_YES;
 	chunk->die = debris_die;
+	chunk->health = 250;
 	gi.linkentity(chunk);
 }
 
@@ -902,13 +914,9 @@ void
 func_object_touch(edict_t *self, edict_t *other, cplane_t *plane,
 		csurface_t *surf /* unused */)
 {
-	if (!self || !other || !plane)
-	{
-		return;
-	}
-
 	/* only squash thing we fall on top of */
-	if (!plane)
+
+	if (!self || !other || !plane)
 	{
 		return;
 	}
@@ -1624,6 +1632,37 @@ commander_body_drop(edict_t *self)
 	self->s.origin[2] += 2;
 }
 
+void
+commander_body_die(edict_t *self, edict_t *inflictor /* unused */,
+		edict_t *attacker /* unused */, int damage, vec3_t point /* unused */)
+{
+	int n;
+
+	if (!self)
+	{
+		return;
+	}
+
+	/* check for gib */
+	if (self->health <= self->gib_health)
+	{
+		gi.sound(self, CHAN_BODY, gi.soundindex("tank/pain.wav"), 1, ATTN_NORM, 0);
+
+		ThrowGib(self, "models/objects/gibs/sm_meat/tris.md2", damage, GIB_ORGANIC);
+
+		for (n = 0; n < 4; n++)
+		{
+			ThrowGib(self, "models/objects/gibs/sm_metal/tris.md2", damage, GIB_METALLIC);
+		}
+
+		ThrowGib(self, "models/objects/gibs/chest/tris.md2", damage, GIB_ORGANIC);
+		ThrowHead(self, "models/objects/gibs/gear/tris.md2", damage, GIB_METALLIC);
+		self->deadflag = DEAD_DEAD;
+
+		return;
+	}
+}
+
 void
 SP_monster_commander_body(edict_t *self)
 {
@@ -1640,8 +1679,19 @@ SP_monster_commander_body(edict_t *self)
 	VectorSet(self->maxs, 32, 32, 48);
 	self->use = commander_body_use;
 	self->takedamage = DAMAGE_YES;
-	self->flags = FL_GODMODE;
 	self->s.renderfx |= RF_FRAMELERP;
+
+	if (g_commanderbody_nogod->value)
+	{
+		self->deadflag = DEAD_DEAD;
+		self->svflags |= SVF_MONSTER | SVF_DEADMONSTER;
+		self->die = commander_body_die;
+	}
+	else
+	{
+		self->flags = FL_GODMODE;
+	}
+
 	gi.linkentity(self);
 
 	gi.soundindex("tank/thud.wav");
@@ -2127,7 +2177,7 @@ SP_misc_gib_arm(edict_t *ent)
 	}
 
 	gi.setmodel(ent, "models/objects/gibs/arm/tris.md2");
-	ent->solid = SOLID_NOT;
+	ent->solid = SOLID_BBOX;
 	ent->s.effects |= EF_GIB;
 	ent->takedamage = DAMAGE_YES;
 	ent->die = gib_die;
@@ -2155,7 +2205,7 @@ SP_misc_gib_leg(edict_t *ent)
 	}
 
 	gi.setmodel(ent, "models/objects/gibs/leg/tris.md2");
-	ent->solid = SOLID_NOT;
+	ent->solid = SOLID_BBOX;
 	ent->s.effects |= EF_GIB;
 	ent->takedamage = DAMAGE_YES;
 	ent->die = gib_die;
@@ -2183,7 +2233,7 @@ SP_misc_gib_head(edict_t *ent)
 	}
 
 	gi.setmodel(ent, "models/objects/gibs/head/tris.md2");
-	ent->solid = SOLID_NOT;
+	ent->solid = SOLID_BBOX;
 	ent->s.effects |= EF_GIB;
 	ent->takedamage = DAMAGE_YES;
 	ent->die = gib_die;
