@@ -64,14 +64,15 @@
  * system and architecture are in the hands of the user.
  */
 
+#include "../../common/header/common.h" // YQ2ARCH
 #include "../header/local.h"
-
+#include "savegame.h"
 /*
  * When ever the savegame version is changed, q2 will refuse to
  * load older savegames. This should be bumped if the files
  * in tables/ are changed, otherwise strange things may happen.
  */
-#define SAVEGAMEVER "YQ2-2"
+#define SAVEGAMEVER "YQ2-5"
 
 #ifndef BUILD_DATE
 #define BUILD_DATE __DATE__
@@ -120,28 +121,6 @@
  #define ARCH_1 "unknown"
 #endif
 
-/*
- * Connects a human readable
- * function signature with
- * the corresponding pointer
- */
-typedef struct
-{
-	char *funcStr;
-	byte *funcPtr;
-} functionList_t;
-
-/*
- * Connects a human readable
- * mmove_t string with the
- * corresponding pointer
- * */
-typedef struct
-{
-	char	*mmoveStr;
-	mmove_t *mmovePtr;
-} mmoveList_t;
-
 /* ========================================================= */
 
 /*
@@ -156,7 +135,7 @@ typedef struct
  * to each of the functions
  * prototyped above.
  */
-functionList_t functionList[] = {
+static functionList_t functionList[] = {
 	#include "tables/gamefunc_list.h"
 };
 
@@ -173,12 +152,12 @@ functionList_t functionList[] = {
  * functions prototyped
  * above.
  */
-mmoveList_t mmoveList[] = {
+static mmoveList_t mmoveList[] = {
 	#include "tables/gamemmove_list.h"
 };
 
 /*
- * Fields to be saved
+ * Fields to be saved (used in g_spawn.c)
  */
 field_t fields[] = {
 	#include "tables/fields.h"
@@ -188,7 +167,7 @@ field_t fields[] = {
  * Level fields to
  * be saved
  */
-field_t levelfields[] = {
+static field_t levelfields[] = {
 	#include "tables/levelfields.h"
 };
 
@@ -196,7 +175,7 @@ field_t levelfields[] = {
  * Client fields to
  * be saved
  */
-field_t clientfields[] = {
+static field_t clientfields[] = {
 	#include "tables/clientfields.h"
 };
 
@@ -232,8 +211,14 @@ InitGame(void)
 	maxspectators = gi.cvar("maxspectators", "4", CVAR_SERVERINFO);
 	deathmatch = gi.cvar("deathmatch", "0", CVAR_LATCH);
 	coop = gi.cvar("coop", "0", CVAR_LATCH);
+	coop_pickup_weapons = gi.cvar("coop_pickup_weapons", "1", CVAR_ARCHIVE);
+	coop_elevator_delay = gi.cvar("coop_elevator_delay", "1.0", CVAR_ARCHIVE);
 	skill = gi.cvar("skill", "1", CVAR_LATCH);
 	maxentities = gi.cvar("maxentities", "1024", CVAR_LATCH);
+	g_footsteps = gi.cvar("g_footsteps", "1", CVAR_ARCHIVE);
+	g_monsterfootsteps = gi.cvar("g_monsterfootsteps", "0", CVAR_ARCHIVE);
+	g_fix_triggered = gi.cvar ("g_fix_triggered", "0", 0);
+	g_commanderbody_nogod = gi.cvar("g_commanderbody_nogod", "0", CVAR_ARCHIVE);
 
 	/* change anytime vars */
 	dmflags = gi.cvar("dmflags", "0", CVAR_SERVERINFO);
@@ -258,6 +243,11 @@ InitGame(void)
 	/* dm map list */
 	sv_maplist = gi.cvar("sv_maplist", "", 0);
 
+	/* others */
+	aimfix = gi.cvar("aimfix", "0", CVAR_ARCHIVE);
+	g_machinegun_norecoil = gi.cvar("g_machinegun_norecoil", "0", CVAR_ARCHIVE);
+	g_swap_speed = gi.cvar("g_swap_speed", "1", 0);
+
 	/* items */
 	InitItems();
 
@@ -733,7 +723,7 @@ WriteClient(FILE *f, gclient_t *client)
  * Read the client struct from a file
  */
 void
-ReadClient(FILE *f, gclient_t *client)
+ReadClient(FILE *f, gclient_t *client, short save_ver)
 {
 	field_t *field;
 
@@ -741,7 +731,14 @@ ReadClient(FILE *f, gclient_t *client)
 
 	for (field = clientfields; field->name; field++)
 	{
-		ReadField(f, field, (byte *)client);
+		if (field->save_ver <= save_ver)
+		{
+			ReadField(f, field, (byte *)client);
+		}
+	}
+	if (save_ver < 3)
+	{
+		InitClientResp(client);
 	}
 }
 
@@ -749,10 +746,10 @@ ReadClient(FILE *f, gclient_t *client)
 
 /*
  * Writes the game struct into
- * a file. This is called when
- * ever the games goes to e new
- * level or the user saves the
- * game. Saved informations are:
+ * a file. This is called whenever
+ * the game goes to a new level or
+ * the user saves the game. The saved
+ * information consists of:
  * - cross level data
  * - client states
  * - help computer info
@@ -760,19 +757,16 @@ ReadClient(FILE *f, gclient_t *client)
 void
 WriteGame(const char *filename, qboolean autosave)
 {
+	savegameHeader_t sv;
 	FILE *f;
 	int i;
-	char str_ver[32];
-	char str_game[32];
-	char str_os[32];
-	char str_arch[32];
 
 	if (!autosave)
 	{
 		SaveClientData();
 	}
 
-	f = fopen(filename, "wb");
+	f = Q_fopen(filename, "wb");
 
 	if (!f)
 	{
@@ -780,20 +774,14 @@ WriteGame(const char *filename, qboolean autosave)
 	}
 
 	/* Savegame identification */
-	memset(str_ver, 0, sizeof(str_ver));
-	memset(str_game, 0, sizeof(str_game));
-	memset(str_os, 0, sizeof(str_os));
-	memset(str_arch, 0, sizeof(str_arch));
+	memset(&sv, 0, sizeof(sv));
 
-	Q_strlcpy(str_ver, SAVEGAMEVER, sizeof(str_ver));
-	Q_strlcpy(str_game, GAMEVERSION, sizeof(str_game));
-	Q_strlcpy(str_os, YQ2OSTYPE, sizeof(str_os));
-	Q_strlcpy(str_arch, YQ2ARCH, sizeof(str_arch));
+	Q_strlcpy(sv.ver, SAVEGAMEVER, sizeof(sv.ver) - 1);
+	Q_strlcpy(sv.game, GAMEVERSION, sizeof(sv.game) - 1);
+	Q_strlcpy(sv.os, YQ2OSTYPE, sizeof(sv.os) - 1);
+	Q_strlcpy(sv.arch, YQ2ARCH, sizeof(sv.arch) - 1);
 
-	fwrite(str_ver, sizeof(str_ver), 1, f);
-	fwrite(str_game, sizeof(str_game), 1, f);
-	fwrite(str_os, sizeof(str_os), 1, f);
-	fwrite(str_arch, sizeof(str_arch), 1, f);
+	fwrite(&sv, sizeof(sv), 1, f);
 
 	game.autosaved = autosave;
 	fwrite(&game, sizeof(game), 1, f);
@@ -815,16 +803,15 @@ WriteGame(const char *filename, qboolean autosave)
 void
 ReadGame(const char *filename)
 {
+	savegameHeader_t sv;
 	FILE *f;
 	int i;
-	char str_ver[32];
-	char str_game[32];
-	char str_os[32];
-	char str_arch[32];
+
+	short save_ver = 0;
 
 	gi.FreeTags(TAG_GAME);
 
-	f = fopen(filename, "rb");
+	f = Q_fopen(filename, "rb");
 
 	if (!f)
 	{
@@ -832,64 +819,88 @@ ReadGame(const char *filename)
 	}
 
 	/* Sanity checks */
-	fread(str_ver, sizeof(str_ver), 1, f);
-	fread(str_game, sizeof(str_game), 1, f);
-	fread(str_os, sizeof(str_os), 1, f);
-	fread(str_arch, sizeof(str_arch), 1, f);
+	fread(&sv, sizeof(sv), 1, f);
 
-	if (!strcmp(str_ver, SAVEGAMEVER))
+	static const struct {
+		const char* verstr;
+		int vernum;
+	} version_mappings[] = {
+		{"YQ2-1", 1},
+		{"YQ2-2", 2},
+		{"YQ2-3", 3},
+		{"YQ2-4", 4},
+		{"YQ2-5", 5},
+	};
+
+	for (i=0; i < sizeof(version_mappings)/sizeof(version_mappings[0]); ++i)
 	{
-		if (strcmp(str_game, GAMEVERSION))
+		if (strcmp(version_mappings[i].verstr, sv.ver) == 0)
+		{
+			save_ver = version_mappings[i].vernum;
+			break;
+		}
+	}
+
+	if (save_ver == 0) // not found in mappings table
+	{
+		fclose(f);
+		gi.error("Savegame from an incompatible version.\n");
+	}
+
+	if (save_ver == 1)
+	{
+		if (strcmp(sv.game, GAMEVERSION) != 0)
 		{
 			fclose(f);
 			gi.error("Savegame from another game.so.\n");
 		}
-		else if (strcmp(str_os, YQ2OSTYPE))
+		else if (strcmp(sv.os, OSTYPE_1) != 0)
 		{
 			fclose(f);
 			gi.error("Savegame from another os.\n");
 		}
-		else if (strcmp(str_arch, YQ2ARCH))
+
+#ifdef _WIN32
+		/* Windows was forced to i386 */
+		if (strcmp(sv.arch, "i386") != 0)
 		{
 			fclose(f);
 			gi.error("Savegame from another architecture.\n");
 		}
+#else
+		if (strcmp(sv.arch, ARCH_1) != 0)
+		{
+			fclose(f);
+			gi.error("Savegame from another architecture.\n");
+		}
+#endif
 	}
-	else if (!strcmp(str_ver, "YQ2-1"))
+	else // all newer savegame versions
 	{
-		if (strcmp(str_game, GAMEVERSION))
+		if (strcmp(sv.game, GAMEVERSION) != 0)
 		{
 			fclose(f);
 			gi.error("Savegame from another game.so.\n");
 		}
-		else if (strcmp(str_os, OSTYPE_1))
+		else if (strcmp(sv.os, YQ2OSTYPE) != 0)
 		{
 			fclose(f);
 			gi.error("Savegame from another os.\n");
 		}
-
-		if (!strcmp(str_os, "Windows"))
+		else if (strcmp(sv.arch, YQ2ARCH) != 0)
 		{
-			/* Windows was forced to i386 */
-			if (strcmp(str_arch, "i386"))
+#if defined(_WIN32) && (defined(__i386__) || defined(_M_IX86))
+			// before savegame version "YQ2-4" (and after version 1),
+			// the official Win32 binaries accidentally had the YQ2ARCH "AMD64"
+			// instead of "i386" set due to a bug in the Makefile.
+			// This quirk allows loading those savegames anyway
+			if (save_ver >= 4 || strcmp(sv.arch, "AMD64") != 0)
+#endif
 			{
 				fclose(f);
 				gi.error("Savegame from another architecture.\n");
 			}
 		}
-		else
-		{
-			if (strcmp(str_arch, ARCH_1))
-			{
-				fclose(f);
-				gi.error("Savegame from another architecture.\n");
-			}
-		}
-	}
-	else
-	{
-		fclose(f);
-		gi.error("Savegame from an incompatible version.\n");
 	}
 
 	g_edicts = gi.TagMalloc(game.maxentities * sizeof(g_edicts[0]), TAG_GAME);
@@ -901,7 +912,7 @@ ReadGame(const char *filename)
 
 	for (i = 0; i < game.maxclients; i++)
 	{
-		ReadClient(f, &game.clients[i]);
+		ReadClient(f, &game.clients[i], save_ver);
 	}
 
 	fclose(f);
@@ -980,7 +991,7 @@ WriteLevel(const char *filename)
 	edict_t *ent;
 	FILE *f;
 
-	f = fopen(filename, "wb");
+	f = Q_fopen(filename, "wb");
 
 	if (!f)
 	{
@@ -1071,7 +1082,7 @@ ReadLevel(const char *filename)
 	int i;
 	edict_t *ent;
 
-	f = fopen(filename, "rb");
+	f = Q_fopen(filename, "rb");
 
 	if (!f)
 	{
